from __future__ import absolute_import, print_function

# Standard modules
import logging
import yaml

from qtpy.QtCore import Qt, QSettings
from qtpy.QtWidgets import (QLineEdit, QTabWidget, QDialog,
                            QScrollArea, QVBoxLayout, QCheckBox,
                            QComboBox, QHBoxLayout, QDialogButtonBox,
                            QLabel, QWidget, QFormLayout)

from ..properties import Paths
from gnuradio import gr

# Logging
log = logging.getLogger(f"grc.application.{__name__}")


class PreferencesDialog(QDialog):
    pref_dict = {}

    def __init__(self, qsettings):
        super(QDialog, self).__init__()
        self.qsettings = qsettings

        self.setMinimumSize(600, 400)
        self.setModal(True)

        self.setWindowTitle("GRC Preferences")
        self.tab_widget = QTabWidget()

        self.rt_prefs = QSettings(gr.userconf_path() + "/config.conf", QSettings.IniFormat)

        with open(Paths.AVAILABLE_PREFS_YML) as available_prefs_yml:
            self.pref_dict = yaml.safe_load(available_prefs_yml)

        self.populate_tabs()

        buttons = QDialogButtonBox.Save | QDialogButtonBox.Cancel
        self.buttonBox = QDialogButtonBox(buttons)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)
        self.layout = QVBoxLayout()
        self.layout.addWidget(self.tab_widget)
        self.layout.addWidget(self.buttonBox)

        self.setLayout(self.layout)

    def new_tab_widget(self) -> QWidget:
        widget = QWidget()
        vbox = QVBoxLayout()
        scroll = QScrollArea()
        scroll.setWidgetResizable(True)
        content = QWidget()
        widget.form = QFormLayout(content)

        widget.setLayout(vbox)
        vbox.addWidget(scroll)
        scroll.setWidget(content)

        return widget

    def populate_tabs(self) -> None:
        for cat in self.pref_dict['categories']:
            tab = cat['tab'] = self.new_tab_widget()
            self.tab_widget.addTab(tab, cat['name'])

            for item in cat['items']:
                full_key = cat['key'] + '/' + item['key']

                item['_label'] = QLabel(item['name'])

                if item['dtype'] == 'bool':
                    item['_edit'] = QCheckBox()

                    if self.qsettings.contains(full_key):
                        value = self.qsettings.value(full_key, False, type=bool)
                        item['_edit'].setChecked(value)
                    else:
                        item['_edit'].setChecked(item['default'])
                        self.qsettings.setValue(full_key, item['default'])

                elif item['dtype'] == 'enum':
                    item['_edit'] = QComboBox()
                    for opt in item['option_labels']:
                        item['_edit'].addItem(opt)
                    index = item['options'].index(self.qsettings.value(full_key, item['default'], type=str))
                    item['_edit'].setCurrentIndex(index)
                else:
                    if self.qsettings.contains(full_key):
                        item['_edit'] = QLineEdit(self.qsettings.value(full_key))
                    else:
                        item['_edit'] = QLineEdit(str(item['default']))
                        self.qsettings.setValue(full_key, item['default'])

                item['_line'] = QHBoxLayout()

                if 'tooltip' in item.keys():
                    item['_label'].setToolTip(item['tooltip'])
                    item['_edit'].setToolTip(item['tooltip'])

                tab.form.addRow(item['_label'], item['_edit'])

        rt_tab = self.new_tab_widget()
        for rt_cat in self.pref_dict['runtime']:
            for item in rt_cat['items']:
                full_key = rt_cat['key'] + '/' + item['key']

                item['_label'] = QLabel(item['name'])

                if item['dtype'] == 'bool':
                    item['_edit'] = QCheckBox()

                    if self.rt_prefs.contains(full_key):
                        value = self.rt_prefs.value(full_key, False, type=bool)
                        item['_edit'].setChecked(value)
                    else:
                        item['_edit'].setChecked(item['default'])
                        self.rt_prefs.setValue(full_key, item['default'])

                elif item['dtype'] == 'enum':
                    item['_edit'] = QComboBox()
                    for opt in item['option_labels']:
                        item['_edit'].addItem(opt)
                    index = item['options'].index(self.rt_prefs.value(full_key, item['default'], type=str))
                    item['_edit'].setCurrentIndex(index)
                else:
                    if self.rt_prefs.contains(full_key):
                        item['_edit'] = QLineEdit(self.rt_prefs.value(full_key))
                    else:
                        item['_edit'] = QLineEdit(str(item['default']))
                        self.rt_prefs.setValue(full_key, item['default'])

                item['_line'] = QHBoxLayout()

                if 'tooltip' in item.keys():
                    item['_label'].setToolTip(item['tooltip'])
                    item['_edit'].setToolTip(item['tooltip'])

                rt_tab.form.addRow(item['_label'], item['_edit'])

        self.tab_widget.addTab(rt_tab, 'Runtime')

    def save_all(self):
        log.debug(f'Writing changes to {self.qsettings.fileName()}')

        for cat in self.pref_dict['categories']:
            for item in cat['items']:
                full_key = cat['key'] + '/' + item['key']

                if item['dtype'] == 'bool':
                    self.qsettings.setValue(full_key, item['_edit'].isChecked())
                elif item['dtype'] == 'enum':
                    self.qsettings.setValue(full_key, item['options'][item['_edit'].currentIndex()])
                else:
                    self.qsettings.setValue(full_key, item['_edit'].text())

        self.qsettings.sync()

        for rt_cat in self.pref_dict['runtime']:
            for item in rt_cat['items']:
                full_key = rt_cat['key'] + '/' + item['key']

                if item['dtype'] == 'bool':
                    self.rt_prefs.setValue(full_key, item['_edit'].isChecked())
                elif item['dtype'] == 'enum':
                    self.rt_prefs.setValue(full_key, item['options'][item['_edit'].currentIndex()])
                else:
                    self.rt_prefs.setValue(full_key, item['_edit'].text())

        self.rt_prefs.sync()



#!/usr/bin/env python3
#
# Copyright 2007-2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
"""
Unit tests for OFDM cyclic prefixer.
"""

from gnuradio import gr, gr_unittest, digital, blocks
import pmt


class test_ofdm_cyclic_prefixer (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_wo_tags_no_rolloff(self):
        " The easiest test: make sure the CP is added correctly. "
        fft_len = 8
        cp_len = 2
        expected_result = [6, 7, 0, 1, 2, 3, 4, 5, 6, 7,
                           6, 7, 0, 1, 2, 3, 4, 5, 6, 7]
        src = blocks.vector_source_c(list(range(fft_len)) * 2, False, fft_len)
        cp = digital.ofdm_cyclic_prefixer(fft_len, fft_len + cp_len)
        sink = blocks.vector_sink_c()
        self.tb.connect(src, cp, sink)
        self.tb.run()
        self.assertEqual(sink.data(), expected_result)

    def test_wo_tags_2s_rolloff(self):
        " No tags, but have a 2-sample rolloff "
        fft_len = 8
        cp_len = 2
        rolloff = 2
        expected_result = [7.0 / 2, 8, 1, 2, 3, 4, 5, 6, 7, 8,  # 1.0/2
                           7.0 / 2 + 1.0 / 2, 8, 1, 2, 3, 4, 5, 6, 7, 8]
        src = blocks.vector_source_c(
            list(
                range(
                    1,
                    fft_len +
                    1)) *
            2,
            False,
            fft_len)
        cp = digital.ofdm_cyclic_prefixer(fft_len, fft_len + cp_len, rolloff)
        sink = blocks.vector_sink_c()
        self.tb.connect(src, cp, sink)
        self.tb.run()
        self.assertEqual(sink.data(), expected_result)

    def test_with_tags_2s_rolloff(self):
        " With tags and a 2-sample rolloff "
        fft_len = 8
        cp_len = 2
        tag_name = "ts_last"
        expected_result = [7.0 / 2, 8, 1, 2, 3, 4, 5, 6, 7, 8,  # 1.0/2
                           7.0 / 2 + 1.0 / 2, 8, 1, 2, 3, 4, 5, 6, 7, 8, 1.0 / 2]
        tag2 = gr.tag_t()
        tag2.offset = 1
        tag2.key = pmt.string_to_symbol("random_tag")
        tag2.value = pmt.from_long(42)
        src = blocks.vector_source_c(
            list(range(1, fft_len + 1)) * 2, False, fft_len, (tag2,))
        cp = digital.ofdm_cyclic_prefixer(
            fft_len, fft_len + cp_len, 2, tag_name)
        sink = blocks.tsb_vector_sink_c(tsb_key=tag_name)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                fft_len,
                2,
                tag_name),
            cp,
            sink)
        self.tb.run()
        self.assertEqual(sink.data()[0], expected_result)
        tags = [gr.tag_to_python(x) for x in sink.tags()]
        tags = sorted([(x.offset, x.key, x.value) for x in tags])
        expected_tags = [
            (fft_len + cp_len, "random_tag", 42)
        ]
        self.assertEqual(tags, expected_tags)

    def test_wo_tags_no_rolloff_multiple_cps(self):
        "Two CP lengths, no rolloff and no tags."
        fft_len = 8
        cp_lengths = (3, 2, 2)
        expected_result = [5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7,  # 1
                           6, 7, 0, 1, 2, 3, 4, 5, 6, 7,    # 2
                           6, 7, 0, 1, 2, 3, 4, 5, 6, 7,    # 3
                           5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7,  # 4
                           6, 7, 0, 1, 2, 3, 4, 5, 6, 7,    # 5
                           ]
        src = blocks.vector_source_c(list(range(fft_len)) * 5, False, fft_len)
        cp = digital.ofdm_cyclic_prefixer(fft_len, cp_lengths)
        sink = blocks.vector_sink_c()
        self.tb.connect(src, cp, sink)
        self.tb.run()
        self.assertEqual(sink.data(), expected_result)

    def test_wo_tags_2s_rolloff_multiple_cps(self):
        "Two CP lengths, 2-sample rolloff and no tags."
        fft_len = 8
        cp_lengths = (3, 2, 2)
        rolloff = 2
        expected_result = [6.0 / 2, 7, 8, 1, 2, 3, 4, 5, 6, 7, 8,  # 1
                           7.0 / 2 + 1.0 / 2, 8, 1, 2, 3, 4, 5, 6, 7, 8,  # 2
                           7.0 / 2 + 1.0 / 2, 8, 1, 2, 3, 4, 5, 6, 7, 8,  # 3
                           6.0 / 2 + 1.0 / 2, 7, 8, 1, 2, 3, 4, 5, 6, 7, 8,  # 4
                           7.0 / 2 + 1.0 / 2, 8, 1, 2, 3, 4, 5, 6, 7, 8  # 5
                           ]
        src = blocks.vector_source_c(
            list(
                range(
                    1,
                    fft_len +
                    1)) *
            5,
            False,
            fft_len)
        cp = digital.ofdm_cyclic_prefixer(fft_len, cp_lengths, rolloff)
        sink = blocks.vector_sink_c()
        self.tb.connect(src, cp, sink)
        self.tb.run()
        self.assertEqual(sink.data(), expected_result)

    def test_with_tags_2s_rolloff_multiples_cps(self):
        "Two CP lengths, 2-sample rolloff and tags."
        fft_len = 8
        cp_lengths = (3, 2, 2)
        rolloff = 2
        tag_name = "ts_last"
        expected_result = [
            6.0 / 2, 7, 8, 1, 2, 3, 4, 5, 6, 7, 8,  # 1
            7.0 / 2 + 1.0 / 2, 8, 1, 2, 3, 4, 5, 6, 7, 8, 1.0 / 2  # Last tail
        ]
        # First test tag
        tag0 = gr.tag_t()
        tag0.offset = 0
        tag0.key = pmt.string_to_symbol("first_tag")
        tag0.value = pmt.from_long(24)
        # Second test tag
        tag1 = gr.tag_t()
        tag1.offset = 1
        tag1.key = pmt.string_to_symbol("second_tag")
        tag1.value = pmt.from_long(42)
        src = blocks.vector_source_c(
            list(range(1, fft_len + 1)) * 2, False, fft_len, (tag0, tag1))
        cp = digital.ofdm_cyclic_prefixer(
            fft_len, cp_lengths, rolloff, tag_name)
        sink = blocks.tsb_vector_sink_c(tsb_key=tag_name)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                fft_len,
                2,
                tag_name),
            cp,
            sink)
        self.tb.run()
        self.assertEqual(sink.data()[0], expected_result)
        tags = [gr.tag_to_python(x) for x in sink.tags()]
        tags = sorted([(x.offset, x.key, x.value) for x in tags])
        expected_tags = [
            (0, "first_tag", 24),
            (fft_len + cp_lengths[0], "second_tag", 42)
        ]
        self.assertEqual(tags, expected_tags)


if __name__ == '__main__':
    gr_unittest.run(test_ofdm_cyclic_prefixer)



#!/usr/bin/env python

from gnuradio import gr
from gnuradio import audio
from gnuradio import trellis, digital, filter, blocks
from gnuradio import eng_notation
import math
import sys
import random
import fsm_utils

try:
    from gnuradio import analog
except ImportError:
    sys.stderr.write("Error: Program requires gr-analog.\n")
    sys.exit(1)


def run_test(f, Kb, bitspersymbol, K, channel, modulation, dimensionality, tot_constellation, N0, seed):
    tb = gr.top_block()
    L = len(channel)

    # TX
    # this for loop is TOO slow in python!!!
    packet = [0] * (K + 2 * L)
    random.seed(seed)
    for i in range(len(packet)):
        packet[i] = random.randint(0, 2**bitspersymbol - 1)  # random symbols
    for i in range(L):  # first/last L symbols set to 0
        packet[i] = 0
        packet[len(packet) - i - 1] = 0
    src = blocks.vector_source_s(packet, False)
    mod = digital.chunks_to_symbols_sf(modulation[1], modulation[0])

    # CHANNEL
    isi = filter.fir_filter_fff(1, channel)
    add = blocks.add_ff()
    noise = analog.noise_source_f(analog.GR_GAUSSIAN, math.sqrt(N0 / 2), seed)

    # RX
    # skip the first L samples since you know they are coming from the L zero symbols
    skip = blocks.skiphead(gr.sizeof_float, L)
    # metrics = trellis.metrics_f(f.O(),dimensionality,tot_constellation,digital.TRELLIS_EUCLIDEAN) # data preprocessing to generate metrics for Viterbi
    # va = trellis.viterbi_s(f,K+L,0,0) # Put -1 if the Initial/Final states are not set.
    # using viterbi_combined_s instead of metrics_f/viterbi_s allows larger packet lengths because metrics_f is complaining for not being able to allocate large buffers. This is due to the large f.O() in this application...
    va = trellis.viterbi_combined_s(
        f, K + L, 0, 0, dimensionality, tot_constellation, digital.TRELLIS_EUCLIDEAN)
    dst = blocks.vector_sink_s()

    tb.connect(src, mod)
    tb.connect(mod, isi, (add, 0))
    tb.connect(noise, (add, 1))
    #tb.connect (add,metrics)
    #tb.connect (metrics,va,dst)
    tb.connect(add, skip, va, dst)

    tb.run()

    data = dst.data()
    ntotal = len(data) - L
    nright = 0
    for i in range(ntotal):
        if packet[i + L] == data[i]:
            nright = nright + 1
        # else:
            # print "Error in ", i

    return (ntotal, ntotal - nright)


def main(args):
    nargs = len(args)
    if nargs == 2:
        esn0_db = float(args[0])
        rep = int(args[1])
    else:
        sys.stderr.write(
            'usage: test_viterbi_equalization1.py Es/No_db  repetitions\n')
        sys.exit(1)

    # system parameters
    Kb = 2048  # packet size in bits
    modulation = fsm_utils.pam4  # see fsm_utlis.py for available predefined modulations
    # see fsm_utlis.py for available predefined test channels
    channel = fsm_utils.c_channel
    # generate the FSM automatically
    f = trellis.fsm(len(modulation[1]), len(channel))
    # bits per FSM input symbol
    bitspersymbol = int(round(math.log(f.I()) / math.log(2)))
    K = Kb / bitspersymbol  # packet size in trellis steps

    # generate the lookup table (normalize energy to 1)
    tot_channel = fsm_utils.make_isi_lookup(modulation, channel, True)
    dimensionality = tot_channel[0]
    tot_constellation = tot_channel[1]
    N0 = pow(10.0, -esn0_db / 10.0)  # noise variance
    if len(tot_constellation) / dimensionality != f.O():
        sys.stderr.write(
            'Incompatible FSM output cardinality and lookup table size.\n')
        sys.exit(1)

    tot_s = 0  # total number of transmitted shorts
    terr_s = 0  # total number of shorts in error
    terr_p = 0  # total number of packets in error

    for i in range(rep):
        (s, e) = run_test(f, Kb, bitspersymbol, K, channel, modulation, dimensionality, tot_constellation,
                          N0, -int(666 + i))  # run experiment with different seed to get different data and noise realizations
        tot_s = tot_s + s
        terr_s = terr_s + e
        terr_p = terr_p + (terr_s != 0)
        if ((i + 1) % 100 == 0):  # display progress
            print(i + 1, terr_p, '%.2e' % ((1.0 * terr_p) / (i + 1)),
                  tot_s, terr_s, '%.2e' % ((1.0 * terr_s) / tot_s))
    # estimate of the (short or symbol) error rate
    print(rep, terr_p, '%.2e' % ((1.0 * terr_p) / (i + 1)),
          tot_s, terr_s, '%.2e' % ((1.0 * terr_s) / tot_s))


if __name__ == '__main__':
    main(sys.argv[1:])



#!/usr/bin/env python
#
# Copyright 2015 Free Software Foundation, Inc.
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import numpy as np
import time
import sys
import copy


def bsc_channel(p):
    '''
    binary symmetric channel (BSC)
    output alphabet Y = {0, 1} and
    W(0|0) = W(1|1) and W(1|0) = W(0|1)

    this function returns a prob's vector for a BSC
    p denotes an erroneous transition
    '''
    if not (p >= 0.0 and p <= 1.0):
        print("given p is out of range!")
        return np.array([], dtype=float)

    # 0 -> 0, 0 -> 1, 1 -> 0, 1 -> 1
    W = np.array([[1 - p, p], [p, 1 - p]], dtype=float)
    return W


def power_of_2_int(num):
    return int(np.log2(num))


def is_power_of_two(num):
    if type(num) != int:
        return False  # make sure we only compute integers.
    return num != 0 and ((num & (num - 1)) == 0)


def bit_reverse(value, n):
    # is this really missing in NumPy???
    seq = np.int_(value)
    rev = np.int_(0)
    rmask = np.int_(1)
    lmask = np.int_(2 ** (n - 1))
    for i in range(n // 2):
        shiftval = n - 1 - (i * 2)
        rshift = np.left_shift(np.bitwise_and(seq, rmask), shiftval)
        lshift = np.right_shift(np.bitwise_and(seq, lmask), shiftval)
        rev = np.bitwise_or(rev, rshift)
        rev = np.bitwise_or(rev, lshift)
        rmask = np.left_shift(rmask, 1)
        lmask = np.right_shift(lmask, 1)
    if not n % 2 == 0:
        rev = np.bitwise_or(rev, np.bitwise_and(seq, rmask))
    return rev


def bit_reverse_vector(vec, n):
    return np.array([bit_reverse(e, n) for e in vec], dtype=vec.dtype)


def get_Bn(n):
    # this is a bit reversal matrix.
    lw = power_of_2_int(n)  # number of used bits
    indexes = [bit_reverse(i, lw) for i in range(n)]
    Bn = np.zeros((n, n), type(n))
    for i, index in enumerate(indexes):
        Bn[i][index] = 1
    return Bn


def get_Fn(n):
    # this matrix defines the actual channel combining.
    if n == 1:
        return np.array([1, ])
    nump = power_of_2_int(n) - 1  # number of Kronecker products to calculate
    F2 = np.array([[1, 0], [1, 1]], np.int_)
    Fn = F2
    for i in range(nump):
        Fn = np.kron(Fn, F2)
    return Fn


def get_Gn(n):
    # this matrix is called generator matrix
    if not is_power_of_two(n):
        print("invalid input")
        return None
    if n == 1:
        return np.array([1, ])
    Bn = get_Bn(n)
    Fn = get_Fn(n)
    Gn = np.dot(Bn, Fn)
    return Gn


def unpack_byte(byte, nactive):
    if np.amin(byte) < 0 or np.amax(byte) > 255:
        return None
    if not byte.dtype == np.uint8:
        byte = byte.astype(np.uint8)
    if nactive == 0:
        return np.array([], dtype=np.uint8)
    return np.unpackbits(byte)[-nactive:]


def pack_byte(bits):
    if len(bits) == 0:
        return 0
    if np.amin(bits) < 0 or np.amax(bits) > 1:  # only '1' and '0' in bits array allowed!
        return None
    bits = np.concatenate((np.zeros(8 - len(bits), dtype=np.uint8), bits))
    res = np.packbits(bits)[0]
    return res


def show_progress_bar(ndone, ntotal):
    nchars = 50

    fract = (1. * ndone / ntotal)
    percentage = 100. * fract
    ndone_chars = int(nchars * fract)
    nundone_chars = nchars - ndone_chars
    sys.stdout.write('\r[{0}{1}] {2:5.2f}% ({3} / {4})'.format('=' *
                     ndone_chars, ' ' * nundone_chars, percentage, ndone, ntotal))


def mutual_information(w):
    '''
    calculate mutual information I(W)
    I(W) = sum over y e Y ( sum over x e X ( ... ) )
    .5 W(y|x) log frac { W(y|x) }{ .5 W(y|0) + .5 W(y|1) }
    '''
    ydim, xdim = np.shape(w)
    i = 0.0
    for y in range(ydim):
        for x in range(xdim):
            v = w[y][x] * np.log2(w[y][x] / (0.5 * w[y][0] + 0.5 * w[y][1]))
            i += v
    i /= 2.0
    return i


def bhattacharyya_parameter(w):
    '''
    bhattacharyya parameter is a measure of similarity between two prob. distributions
    THEORY: sum over all y e Y for sqrt( W(y|0) * W(y|1) )
    Implementation:
    Numpy vector of dimension (2, mu//2)
    holds probabilities P(x|0), first vector for even, second for odd.
    '''
    dim = np.shape(w)
    if len(dim) != 2:
        raise ValueError

    if dim[0] > dim[1]:
        raise ValueError

    z = np.sum(np.sqrt(w[0] * w[1]))
    # need all
    return z


def main():
    print('helper functions')

    for i in range(9):
        print(i, 'is power of 2: ', is_power_of_two(i))
    n = 6
    m = 2 ** n

    pos = np.arange(m)
    rev_pos = bit_reverse_vector(pos, n)
    print(pos)
    print(rev_pos)

    f = np.linspace(.01, .29, 10)
    e = np.linspace(.03, .31, 10)

    b = np.array([e, f])
    zp = bhattacharyya_parameter(b)
    print(zp)

    a = np.sum(np.sqrt(e * f))
    print(a)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# Copyright 2021 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, gr_unittest, blocks, network
import socket
import threading
import time


class qa_tcp_sink (gr_unittest.TestCase):
    def tcp_receive(self, serversocket):
        for _ in range(2):
            clientsocket, address = serversocket.accept()
            while True:
                data = clientsocket.recv(4096)
                if not data:
                    break
            clientsocket.close()

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_restart(self):
        serversocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        serversocket.settimeout(30.0)
        serversocket.bind(('localhost', 2000))
        serversocket.listen()

        thread = threading.Thread(target=self.tcp_receive, args=(serversocket,))
        thread.start()

        null_source = blocks.null_source(gr.sizeof_gr_complex)
        throttle = blocks.throttle(gr.sizeof_gr_complex, 320000, True)
        tcp_sink = network.tcp_sink(gr.sizeof_gr_complex, 1, '127.0.0.1', 2000, 1)
        self.tb.connect(null_source, throttle, tcp_sink)
        self.tb.start()
        time.sleep(0.1)
        self.tb.stop()
        time.sleep(0.1)
        self.tb.start()
        time.sleep(0.1)
        self.tb.stop()

        thread.join()
        serversocket.close()


if __name__ == '__main__':
    gr_unittest.run(qa_tcp_sink)



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, digital, blocks

default_access_code = '\xAC\xDD\xA4\xE2\xF2\x8C\x20\xFC'


def string_to_1_0_list(s):
    r = []
    for ch in s:
        x = ord(ch)
        for i in range(8):
            t = (x >> i) & 0x1
            r.append(t)
    return r


def to_1_0_string(L):
    return ''.join([chr(x + ord('0')) for x in L])


class test_framer_sink(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):

        code = (1, 1, 0, 1)
        access_code = to_1_0_string(code)
        header = tuple(2 * [0, 0, 0, 0, 0, 0, 0, 0, 0,
                            0, 0, 0, 0, 0, 0, 1])  # len=1
        pad = (0,) * 100
        src_data = code + header + (0, 1, 0, 0, 0, 0, 0, 1) + pad
        expected_data = b'A'

        rcvd_pktq = gr.msg_queue()

        src = blocks.vector_source_b(src_data)
        correlator = digital.correlate_access_code_bb(access_code, 0)
        framer_sink = digital.framer_sink_1(rcvd_pktq)
        vsnk = blocks.vector_sink_b()

        self.tb.connect(src, correlator, framer_sink)
        self.tb.connect(correlator, vsnk)
        self.tb.run()

        result_data = rcvd_pktq.delete_head()
        result_data = result_data.to_string()
        self.assertEqual(expected_data, result_data)

    def test_002(self):

        code = tuple(string_to_1_0_list(default_access_code))
        access_code = to_1_0_string(code)
        header = tuple(2 * [0, 0, 0, 0, 0, 0, 0, 0, 0,
                            0, 0, 0, 0, 0, 1, 0])  # len=2
        pad = (0,) * 100
        src_data = code + header + \
            (0, 1, 0, 0, 1, 0, 0, 0) + (0, 1, 0, 0, 1, 0, 0, 1) + pad
        expected_data = b'HI'

        rcvd_pktq = gr.msg_queue()

        src = blocks.vector_source_b(src_data)
        correlator = digital.correlate_access_code_bb(access_code, 0)
        framer_sink = digital.framer_sink_1(rcvd_pktq)
        vsnk = blocks.vector_sink_b()

        self.tb.connect(src, correlator, framer_sink)
        self.tb.connect(correlator, vsnk)
        self.tb.run()

        result_data = rcvd_pktq.delete_head()
        result_data = result_data.to_string()
        self.assertEqual(expected_data, result_data)


if __name__ == '__main__':
    gr_unittest.run(test_framer_sink)



#!/usr/bin/env python
#
# Copyright 2004,2007,2010,2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, filter, blocks

import math


class test_interp_fir_filter(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_fff(self):
        taps = [1, 10, 100, 1000, 10000]
        src_data = (0, 2, 3, 5, 7, 11, 13, 17)
        interpolation = 3
        xr = (0, 0, 0,
              2, 20, 200, 2003, 20030,
              300, 3005, 30050,
              500, 5007, 50070,
              700, 7011, 70110,
              1100, 11013, 110130,
              1300, 13017, 130170)
        expected_result = [float(x) for x in xr]

        src = blocks.vector_source_f(src_data)
        op = filter.interp_fir_filter_fff(interpolation, taps)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        L = min(len(result_data), len(expected_result))
        self.assertEqual(expected_result[0:L], result_data[0:L])


if __name__ == '__main__':
    gr_unittest.run(test_interp_fir_filter)



#!/usr/bin/env python
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import numpy as np

from gnuradio import gr, gr_unittest, blocks
from gnuradio import fec
from gnuradio.fec import extended_encoder
from gnuradio.fec import extended_decoder

from _qa_helper import _qa_helper
from _qa_helper_async import _qa_helper_async


class test_fecapi_dummy(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_parallelism0_00(self):
        frame_size = 30
        enc = fec.dummy_encoder_make(frame_size * 8)
        dec = fec.dummy_decoder.make(frame_size * 8)
        threading = None
        self.test = _qa_helper(10 * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()

        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_parallelism0_01(self):
        frame_size = 30
        enc = fec.dummy_encoder_make(frame_size * 8)
        dec = fec.dummy_decoder.make(frame_size * 8)
        threading = 'ordinary'
        self.test = _qa_helper(10 * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()

        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_parallelism0_02(self):
        frame_size = 30
        enc = fec.dummy_encoder_make(frame_size * 8)
        dec = fec.dummy_decoder.make(frame_size * 8)
        threading = 'capillary'
        self.test = _qa_helper(10 * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()

        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_parallelism1_00(self):
        frame_size = 30
        enc = list(
            map((lambda a: fec.dummy_encoder_make(frame_size * 8)), list(range(0, 1))))
        dec = list(
            map((lambda a: fec.dummy_decoder.make(frame_size * 8)), list(range(0, 1))))
        threading = None
        self.test = _qa_helper(10 * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()

        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_parallelism1_01(self):
        frame_size = 30
        enc = list(
            map((lambda a: fec.dummy_encoder_make(frame_size * 8)), list(range(0, 1))))
        dec = list(
            map((lambda a: fec.dummy_decoder.make(frame_size * 8)), list(range(0, 1))))
        threading = 'ordinary'
        self.test = _qa_helper(10 * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()

        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_parallelism1_02(self):
        frame_size = 300
        enc = list(
            map((lambda a: fec.dummy_encoder_make(frame_size * 8)), list(range(0, 1))))
        dec = list(
            map((lambda a: fec.dummy_decoder.make(frame_size * 8)), list(range(0, 1))))
        threading = 'capillary'
        self.test = _qa_helper(10 * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()
        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_parallelism1_03(self):
        frame_size = 30
        dims = 10
        enc = list(map((lambda a: fec.dummy_encoder_make(
            frame_size * 8)), list(range(0, dims))))
        dec = list(map((lambda a: fec.dummy_decoder.make(
            frame_size * 8)), list(range(0, dims))))
        threading = 'ordinary'
        self.test = _qa_helper(dims * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()

        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_parallelism1_04(self):
        frame_size = 30
        dims = 16
        enc = list(map((lambda a: fec.dummy_encoder_make(
            frame_size * 8)), list(range(0, dims))))
        dec = list(map((lambda a: fec.dummy_decoder.make(
            frame_size * 8)), list(range(0, dims))))
        threading = 'capillary'
        self.test = _qa_helper(dims * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()

        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_parallelism1_05(self):
        frame_size = 30
        dims = 5
        enc = list(map((lambda a: fec.dummy_encoder_make(
            frame_size * 8)), list(range(0, dims))))
        #dec = list(map((lambda a: fec.dummy_decoder.make(frame_size*8)), range(0,dims)))
        threading = 'capillary'

        self.assertRaises(
            AttributeError,
            lambda: extended_encoder(
                enc,
                threading=threading,
                puncpat="11"))

    def test_parallelism1_06(self):
        frame_size = 30
        dims = 5
        #enc = list(map((lambda a: fec.dummy_encoder_make(frame_size*8)), range(0,dims)))
        dec = list(map((lambda a: fec.dummy_decoder.make(
            frame_size * 8)), list(range(0, dims))))
        threading = 'capillary'

        self.assertRaises(
            AttributeError,
            lambda: extended_decoder(
                dec,
                threading=threading,
                puncpat="11"))

    def test_parallelism2_00(self):
        frame_size = 30
        dims1 = 16
        dims2 = 16
        enc = list(map((lambda b: list(map((lambda a: fec.dummy_encoder_make(
            frame_size * 8)), list(range(0, dims1))))), list(range(0, dims2))))
        #dec = list(map((lambda b: map((lambda a: fec.dummy_decoder_make(frame_size*8)), range(0,dims1))), range(0,dims2)))
        threading = 'capillary'

        self.assertRaises(
            AttributeError,
            lambda: extended_encoder(
                enc,
                threading=threading,
                puncpat="11"))

    def test_parallelism2_01(self):
        frame_size = 30
        dims1 = 16
        dims2 = 16
        dec = list(map((lambda b: list(map((lambda a: fec.dummy_decoder_make(
            frame_size * 8)), list(range(0, dims1))))), list(range(0, dims2))))
        threading = 'capillary'

        self.assertRaises(
            AttributeError,
            lambda: extended_decoder(
                dec,
                threading=threading,
                puncpat="11"))

    def test_extended_pack_data(self):
        # test if extended encoder gets correct values for input and output
        # conversion.
        n_frames = 10
        frame_size = 32

        data = np.random.randint(0, 2, n_frames * frame_size)
        data.dtype = np.uint8
        packed_data = np.packbits(data)

        tb = gr.top_block()

        src = blocks.vector_source_b(data)
        snk0 = blocks.vector_sink_b(1)
        snk1 = blocks.vector_sink_b(1)
        snk2 = blocks.vector_sink_b(1)
        snk3 = blocks.vector_sink_b(1)

        packer = blocks.pack_k_bits_bb(8)
        tb.connect(src, packer, snk0)

        enc_unpacked = fec.dummy_encoder_make(frame_size, False, False)
        ext_enc_unp = extended_encoder(
            enc_unpacked, threading='none', puncpat='11')
        tb.connect(src, ext_enc_unp, snk1)

        enc_pack = fec.dummy_encoder_make(frame_size // 8, True, False)
        ext_enc_pack = extended_encoder(
            enc_pack, threading='none', puncpat='11')
        tb.connect(src, ext_enc_pack, snk2)

        enc_packed_bits = fec.dummy_encoder_make(frame_size // 8, False, True)
        ext_enc_packed_bits = extended_encoder(
            enc_packed_bits, threading='none', puncpat='11')
        tb.connect(packer, ext_enc_packed_bits, snk3)

        tb.run()

        r0 = snk0.data()
        r1 = snk1.data()
        r2 = snk2.data()
        r3 = snk3.data()

        data = list(data)
        packed_data = list(packed_data)
        self.assertSequenceEqualGR(packed_data, r0)
        self.assertSequenceEqualGR(data, r1)
        self.assertSequenceEqualGR(packed_data, r2)
        self.assertSequenceEqualGR(data, r3)

    def test_async_00(self):
        frame_size = 30
        enc = fec.dummy_encoder_make(frame_size * 8)
        dec = fec.dummy_decoder.make(frame_size * 8)
        for packed in [True, False]:
            for rev_pack in [True, False]:
                with self.subTest(packed=packed, rev_pack=rev_pack):
                    self.test = _qa_helper_async(
                        frame_size, enc, dec, packed, rev_pack)
                    self.test.run()

                    data_in = self.test.snk_input.data()
                    data_out = self.test.snk_output.data()

                    self.assertSequenceEqualGR(data_in, data_out)


if __name__ == '__main__':
    gr_unittest.run(test_fecapi_dummy)



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import random
import numpy

from gnuradio import gr, gr_unittest, blocks, analog, channels
from gnuradio import digital
from gnuradio.digital.utils import tagged_streams
from gnuradio.digital.ofdm_txrx import ofdm_tx


def make_bpsk_burst(fft_len, cp_len, num_bits):
    """ Create a burst of a sync symbol and some BPSK bits """
    sync_symbol = [
        (random.randint(0, 1) * 2) - 1
        for x in range(fft_len // 2)
    ] * 2
    sync_symbols = sync_symbol[-cp_len:] + sync_symbol
    mod_symbols = [
        (random.randint(0, 1) * 2) - 1
        for x in range(num_bits)
    ]
    return sync_symbols + mod_symbols


class qa_ofdm_sync_sc_cfb (gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_detect(self):
        """ Send two bursts, with zeros in between, and check
        they are both detected at the correct position and no
        false alarms occur """
        n_zeros = 15
        fft_len = 32
        cp_len = 4
        sig_len = (fft_len + cp_len) * 10
        tx_signal = [0, ] * n_zeros + make_bpsk_burst(fft_len, cp_len, sig_len)
        tx_signal = tx_signal * 2
        add = blocks.add_cc()
        sync = digital.ofdm_sync_sc_cfb(fft_len, cp_len)
        sink_freq = blocks.vector_sink_f()
        sink_detect = blocks.vector_sink_b()
        self.tb.connect(blocks.vector_source_c(tx_signal), (add, 0))
        self.tb.connect(
            analog.noise_source_c(
                analog.GR_GAUSSIAN, .01), (add, 1))
        self.tb.connect(add, sync)
        self.tb.connect((sync, 0), sink_freq)
        self.tb.connect((sync, 1), sink_detect)
        self.tb.run()
        sig1_detect = sink_detect.data()[0:len(tx_signal) // 2]
        sig2_detect = sink_detect.data()[len(tx_signal) // 2:]
        self.assertAlmostEqual(sig1_detect.index(1), n_zeros + fft_len + cp_len, delta=cp_len - 1)
        self.assertAlmostEqual(sig2_detect.index(1), n_zeros + fft_len + cp_len, delta=cp_len - 1)
        self.assertEqual(numpy.sum(sig1_detect), 1)
        self.assertEqual(numpy.sum(sig2_detect), 1)

    def test_002_freq(self):
        """ Add a fine frequency offset and see if that gets detected properly """
        fft_len = 32
        cp_len = 4
        # This frequency offset is normalized to rads, i.e. \pi == f_s/2
        max_freq_offset = 2 * numpy.pi / fft_len  # Otherwise, it's coarse
        freq_offset = ((2 * random.random()) - 1) * max_freq_offset
        sig_len = (fft_len + cp_len) * 10
        tx_signal = make_bpsk_burst(fft_len, cp_len, sig_len)
        sync = digital.ofdm_sync_sc_cfb(fft_len, cp_len, True)
        sink_freq = blocks.vector_sink_f()
        sink_detect = blocks.vector_sink_b()
        channel = channels.channel_model(0.005, freq_offset / 2.0 / numpy.pi)
        self.tb.connect(blocks.vector_source_c(tx_signal), channel, sync)
        self.tb.connect((sync, 0), sink_freq)
        self.tb.connect((sync, 1), sink_detect)
        self.tb.run()
        phi_hat = sink_freq.data()[sink_detect.data().index(1)]
        est_freq_offset = 2 * phi_hat / fft_len
        self.assertAlmostEqual(est_freq_offset, freq_offset, places=2)

    def test_003_multiburst(self):
        """ Send several bursts, see if the number of detects is correct.
        Burst lengths and content are random.

        The channel is assumed AWGN for this test.
        """
        n_bursts = 42
        fft_len = 32
        cp_len = 4
        tx_signal = []
        for _ in range(n_bursts):
            gap = [0, ] * random.randint(0, 2 * fft_len)
            tx_signal += gap + \
                make_bpsk_burst(fft_len, cp_len, fft_len *
                                random.randint(5, 23))
        # Very loose definition of SNR here
        snr = 20  # dB
        sigma = 10**(-snr / 10)
        # Add noise -- we don't use the channel model blocks, we want to keep
        # this test as self-contained as possible, and all randomness should
        # derive from random.seed() above
        def complex_randn(N): return (numpy.random.randn(
            N) + 1j * numpy.random.randn(N)) * sigma / numpy.sqrt(2)
        tx_signal += complex_randn(len(tx_signal))
        sync = digital.ofdm_sync_sc_cfb(fft_len, cp_len)
        sink_freq = blocks.vector_sink_f()
        sink_detect = blocks.vector_sink_b()
        self.tb.connect(blocks.vector_source_c(tx_signal), sync)
        self.tb.connect((sync, 0), sink_freq)
        self.tb.connect((sync, 1), sink_detect)
        self.tb.run()
        n_bursts_detected = numpy.sum(sink_detect.data())
        self.assertEqual(
            n_bursts_detected, n_bursts,
            msg="Detection error (missed bursts): {}".format(
                (numpy.sum(sink_detect.data()) - n_bursts))
        )

    def test_004_ofdm_packets(self):
        """
        Send several bursts using ofdm_tx, see if the number of detects is correct.
        Burst lengths and content are random.
        """
        n_bursts = 42
        fft_len = 64
        cp_len = 16
        # Here, coarse freq offset is allowed
        max_freq_offset = 2 * numpy.pi / fft_len * 4
        freq_offset = ((2 * random.random()) - 1) * max_freq_offset
        packets = []
        tagname = "packet_length"
        min_packet_length = 10
        max_packet_length = 50
        for _ in range(n_bursts):
            packet_length = random.randint(min_packet_length,
                                           max_packet_length + 1)
            packet = [random.randint(0, 255) for i in range(packet_length)]
            packets.append(packet)
        data, tags = tagged_streams.packets_to_vectors(
            packets, tagname, vlen=1)
        src = blocks.vector_source_b(data, False, 1, tags)
        mod = ofdm_tx(packet_length_tag_key=tagname)
        sync = digital.ofdm_sync_sc_cfb(fft_len, cp_len)
        sink_freq = blocks.vector_sink_f()
        sink_detect = blocks.vector_sink_b()
        noise_level = 0.005
        channel = channels.channel_model(
            noise_level, freq_offset / 2 / numpy.pi)
        self.tb.connect(src, mod, channel, sync, sink_freq)
        self.tb.connect((sync, 1), sink_detect)
        self.tb.run()
        self.assertEqual(numpy.sum(sink_detect.data()), n_bursts)


if __name__ == '__main__':
    gr_unittest.run(qa_ofdm_sync_sc_cfb)



#!/usr/bin/env python
#
# Copyright 2012-2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_probe_signal(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        value = 12.3
        repeats = 100
        src_data = [value] * repeats

        src = blocks.vector_source_f(src_data)
        dst = blocks.probe_signal_f()

        self.tb.connect(src, dst)
        self.tb.run()
        output = dst.level()
        self.assertAlmostEqual(value, output, places=6)

    def test_002(self):
        vector_length = 10
        repeats = 10
        value = [0.5 + i for i in range(0, vector_length)]
        src_data = value * repeats

        src = blocks.vector_source_f(src_data)
        s2v = blocks.stream_to_vector(gr.sizeof_float, vector_length)
        dst = blocks.probe_signal_vf(vector_length)

        self.tb.connect(src, s2v, dst)
        self.tb.run()
        output = dst.level()
        self.assertEqual(len(output), vector_length)
        self.assertAlmostEqual(value[3], output[3], places=6)

    def test_003_race_condition_regression_test(self):
        src = blocks.vector_source_c([1 + 2j, 3 + 4j], True)
        dst = blocks.probe_signal_c()

        self.tb.connect(src, dst)
        self.tb.start()

        while dst.level() == 0.0:
            continue

        for _ in range(100000):
            output = dst.level()
            self.assertIn(output, [1 + 2j, 3 + 4j])

        self.tb.stop()
        self.tb.wait()

    def test_004_race_condition_regression_test_vector(self):
        vector_length = 10
        src_data = [1.0] * vector_length + [2.0] * vector_length

        src = blocks.vector_source_f(src_data, True, vector_length)
        dst = blocks.probe_signal_vf(vector_length)

        self.tb.connect(src, dst)
        self.tb.start()

        while dst.level()[0] == 0.0:
            continue

        for _ in range(10000):
            output = dst.level()
            self.assertIn(output[0], [1.0, 2.0])
            for i in range(1, vector_length):
                self.assertEqual(output[0], output[i])

        self.tb.stop()
        self.tb.wait()


if __name__ == '__main__':
    gr_unittest.run(test_probe_signal)



#!/usr/bin/env python
#
# Copyright 2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import sys
import subprocess
import re
import signal
import time
import atexit
import os
from gnuradio import gr


class monitor(object):
    def __init__(self, tool="gr-ctrlport-monitor"):
        print("ControlPort Monitor starting.")
        self.started = False
        self.tool = tool
        atexit.register(self.shutdown)

        # setup export prefs
        gr.prefs().set_bool("ControlPort", "on", True)
        gr.prefs().set_bool("PerfCounters", "on", True)
        gr.prefs().set_bool("PerfCounters", "export", True)
        if tool == "gr-perf-monitorx":
            gr.prefs().set_bool("ControlPort", "edges_list", True)

    def start(self):
        try:
            self.elem = gr.rpcmanager.get()
            ep = self.elem.endpoints()
            print(f"monitor::endpoints() =  {ep}")

            cmd = [
                self.tool,
                re.search(r"-h (\S+|\d+\.\d+\.\d+\.\d+)", ep[0]).group(1),
                re.search(r"-p (\d+)", ep[0]).group(1),
            ]
            print("running: %s" % (str(cmd)))
            self.proc = subprocess.Popen(cmd)
            self.started = True
        except:
            self.proc = None
            print(
                "Failed to to connect to ControlPort. Please make sure that you have Thrift installed and check your firewall rules."
            )

    def stop(self):
        if self.proc:
            if self.proc.returncode == None:
                print("\tcalling stop on shutdown")
                self.proc.terminate()
        else:
            print("\tno proc to shut down, exiting")

    def shutdown(self):
        print("ctrlport monitor received shutdown signal")
        if self.started:
            self.stop()



#!/usr/bin/env python
#
# Copyright 2019,2020 Free Software Foundation, Inc.
# Copyright 2024 Marcus Müller
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest
import os


class test_prefs (gr_unittest.TestCase):

    def test_001(self):
        p = gr.prefs()

        # Read some options
        self.assertFalse(p.has_option('doesnt', 'exist'))

        # At the time these tests are run, there is not necessarily a default
        # configuration on the build system, so not much to do with testing
        # here

    def _test_env(self, env: str, subdir: [str, None] = None):
        import tempfile
        oldenv = os.getenv(env, "")
        tmpdir = tempfile.mkdtemp()
        if subdir:
            os.mkdir(os.path.join(tmpdir, subdir))
            conffile = os.path.join(tmpdir, subdir, "config.conf")
        else:
            conffile = os.path.join(tmpdir, "config.conf")
        f = open(conffile, "w", encoding="utf-8")
        f.write("[TEST_SECTION]\ncanary=alive\n")
        f.close()
        os.environ[env] = tmpdir
        prefs = gr.prefs()
        self.assertEqual(prefs.get_string("TEST_SECTION", "canary", "verydead"), "alive")
        magicvalue = 1337
        prefs.set_long("WRITE_SECTION", "value", magicvalue)
        self.assertEqual(prefs.get_long("WRITE_SECTION", "value", 0), magicvalue)
        self.assertEqual(prefs.get_string("TEST_SECTION", "canary", "verydead"), "alive")
        os.unlink(conffile)
        if subdir:
            os.rmdir(os.path.join(tmpdir, subdir))
        os.rmdir(tmpdir)
        if oldenv:
            os.putenv(env, oldenv)
        else:
            os.unsetenv(env)

    def test_002_react_to_GR_PREFS_PATH(self):
        self._test_env("GR_PREFS_PATH")

    def test_003_react_to_XDG_CONFIG_HOME(self):
        oldgrprefspath = os.getenv("GR_PREFS_PATH", None)
        if oldgrprefspath:
            os.unsetenv("GR_PREFS_PATH")
        self._test_env("XDG_CONFIG_HOME", "gnuradio")
        if oldgrprefspath:
            os.putenv("GR_PREFS_PATH", oldgrprefspath)


if __name__ == '__main__':
    gr_unittest.run(test_prefs)



#!/usr/bin/env python
#
# Copyright 2006,2007,2010,2011,2013,2017 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, digital, blocks

default_access_code = '\xAC\xDD\xA4\xE2\xF2\x8C\x20\xFC'


def string_to_1_0_list(s):
    r = []
    for ch in s:
        x = ord(ch)
        for i in range(8):
            t = (x >> i) & 0x1
            r.append(t)

    return r


def to_1_0_string(L):
    return ''.join([chr(x + ord('0')) for x in L])


class test_correlate_access_code(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_bb(self):
        pad = (0,) * 64
        src_data = (1, 0, 1, 1, 1, 1, 0, 1, 1) + pad + (0,) * 7
        src = blocks.vector_source_b(src_data)
        op = digital.correlate_access_code_tag_bb("1011", 0, "sync")
        dst = blocks.tag_debug(gr.sizeof_char, "sync")
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.current_tags()
        self.assertEqual(len(result_data), 2)
        self.assertEqual(result_data[0].offset, 4)
        self.assertEqual(result_data[1].offset, 9)

    def test_bb_skip_prefix(self):
        pad = (0,) * 64
        src_data = (0, 1, 1, 1, 1, 0, 0, 1, 1) + pad + (0,) * 7
        src = blocks.vector_source_b(src_data)
        op = digital.correlate_access_code_tag_bb("0011", 0, "sync")
        dst = blocks.tag_debug(gr.sizeof_char, "sync")
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.current_tags()
        self.assertEqual(len(result_data), 1)
        self.assertEqual(result_data[0].offset, 9)

    def test_bb_immediate(self):
        """Test that packets at start of stream match"""
        pad = (0,) * 64
        src_data = (0, 0, 1, 1) + pad + (0,) * 7
        src = blocks.vector_source_b(src_data)
        op = digital.correlate_access_code_tag_bb("0011", 0, "sync")
        dst = blocks.tag_debug(gr.sizeof_char, "sync")
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.current_tags()
        self.assertEqual(len(result_data), 1)
        self.assertEqual(result_data[0].offset, 4)

    def test_002(self):
        code = tuple(string_to_1_0_list(default_access_code))
        access_code = to_1_0_string(code)
        pad = (0,) * 64
        # print code
        # print access_code
        src_data = code + (1, 0, 1, 1) + pad
        src = blocks.vector_source_b(src_data)
        op = digital.correlate_access_code_tag_bb(access_code, 0, "sync")
        dst = blocks.tag_debug(gr.sizeof_char, "sync")
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.current_tags()
        self.assertEqual(len(result_data), 1)
        self.assertEqual(result_data[0].offset, len(code))

    def test_ff(self):
        pad = (0,) * 64
        src_bits = (1, 0, 1, 1, 1, 1, 0, 1, 1) + pad + (0,) * 7
        src_data = [2.0 * x - 1.0 for x in src_bits]
        src = blocks.vector_source_f(src_data)
        op = digital.correlate_access_code_tag_ff("1011", 0, "sync")
        dst = blocks.tag_debug(gr.sizeof_float, "sync")
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.current_tags()
        self.assertEqual(len(result_data), 2)
        self.assertEqual(result_data[0].offset, 4)
        self.assertEqual(result_data[1].offset, 9)

    def test_ff_skip_prefix(self):
        pad = (0,) * 64
        src_bits = (0, 1, 1, 1, 1, 0, 0, 1, 1) + pad + (0,) * 7
        src_data = [2.0 * x - 1.0 for x in src_bits]
        src = blocks.vector_source_f(src_data)
        op = digital.correlate_access_code_tag_ff("0011", 0, "sync")
        dst = blocks.tag_debug(gr.sizeof_float, "sync")
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.current_tags()
        self.assertEqual(len(result_data), 1)
        self.assertEqual(result_data[0].offset, 9)

    def test_ff_immediate(self):
        """Test that packets at start of stream match"""
        pad = (0,) * 64
        src_bits = (0, 0, 1, 1) + pad + (0,) * 7
        src_data = [2.0 * x - 1.0 for x in src_bits]
        src = blocks.vector_source_f(src_data)
        op = digital.correlate_access_code_tag_ff("0011", 0, "sync")
        dst = blocks.tag_debug(gr.sizeof_float, "sync")
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.current_tags()
        self.assertEqual(len(result_data), 1)
        self.assertEqual(result_data[0].offset, 4)

    def test_004(self):
        code = tuple(string_to_1_0_list(default_access_code))
        access_code = to_1_0_string(code)
        pad = (0,) * 64
        # print code
        # print access_code
        src_bits = code + (1, 0, 1, 1) + pad
        src_data = [2.0 * x - 1.0 for x in src_bits]
        src = blocks.vector_source_f(src_data)
        op = digital.correlate_access_code_tag_ff(access_code, 0, "sync")
        dst = blocks.tag_debug(gr.sizeof_float, "sync")
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.current_tags()
        self.assertEqual(len(result_data), 1)
        self.assertEqual(result_data[0].offset, len(code))


if __name__ == '__main__':
    gr_unittest.run(test_correlate_access_code)



#!/usr/bin/env python
#
# Copyright 2007,2010,2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, digital, blocks


class test_pn_correlator_cc(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000_make(self):
        c = digital.pn_correlator_cc(10)

    def test_001_correlate(self):
        degree = 10
        length = 2**degree - 1
        src = digital.glfsr_source_f(degree)
        head = blocks.head(gr.sizeof_float, length * length)
        f2c = blocks.float_to_complex()
        corr = digital.pn_correlator_cc(degree)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, head, f2c, corr, dst)
        self.tb.run()
        data = dst.data()
        self.assertEqual(data[-1], (1.0 + 0j))


if __name__ == '__main__':
    gr_unittest.run(test_pn_correlator_cc)



#!/usr/bin/env python
#
# Copyright 2006,2007,2010,2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from math import pi, cos

from gnuradio import gr, gr_unittest, fft, blocks


class test_goertzel(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def make_tone_data(self, rate, freq):
        return [cos(2 * pi * x * freq / rate) for x in range(rate)]

    def transform(self, src_data, rate, freq):
        src = blocks.vector_source_f(src_data, False)
        dft = fft.goertzel_fc(rate, rate, freq)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, dft, dst)
        self.tb.run()
        return dst.data()

    def test_001(self):  # Measure single tone magnitude
        rate = 8000
        freq = 100
        bin = freq
        src_data = self.make_tone_data(rate, freq)
        expected_result = 0.5
        actual_result = abs(self.transform(src_data, rate, bin)[0])
        self.assertAlmostEqual(expected_result, actual_result, places=4)

    def test_002(self):  # Measure off frequency magnitude
        rate = 8000
        freq = 100
        bin = freq / 2
        src_data = self.make_tone_data(rate, freq)
        expected_result = 0.0
        actual_result = abs(self.transform(src_data, rate, bin)[0])
        self.assertAlmostEqual(expected_result, actual_result, places=4)


if __name__ == '__main__':
    gr_unittest.run(test_goertzel)



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import numpy
import sys
from matplotlib import pyplot
from gnuradio import digital
from .soft_dec_lut_gen import soft_dec_table, calc_soft_dec_from_table, calc_soft_dec
from .psk_constellations import psk_4_0, psk_4_1, psk_4_2, psk_4_3, psk_4_4, psk_4_5, psk_4_6, psk_4_7, sd_psk_4_0, sd_psk_4_1, sd_psk_4_2, sd_psk_4_3, sd_psk_4_4, sd_psk_4_5, sd_psk_4_6, sd_psk_4_7
from .qam_constellations import qam_16_0, sd_qam_16_0


def test_qpsk(i, sample, prec):
    qpsk_const_list = [psk_4_0, psk_4_1, psk_4_2, psk_4_3,
                       psk_4_4, psk_4_5, psk_4_6, psk_4_7]
    qpsk_lut_gen_list = [sd_psk_4_0, sd_psk_4_1, sd_psk_4_2, sd_psk_4_3,
                         sd_psk_4_4, sd_psk_4_5, sd_psk_4_6, sd_psk_4_7]

    constel, code = qpsk_const_list[i]()
    qpsk_lut_gen = qpsk_lut_gen_list[i]

    rot_sym = 1
    side = 2
    width = 2
    c = digital.constellation_rect(constel, code, rot_sym,
                                   side, side, width, width)

    # Get max energy/symbol in constellation
    constel = c.points()
    Es = max([numpy.sqrt(constel_i.real**2 + constel_i.imag**2)
             for constel_i in constel])

    #table = soft_dec_table_generator(qpsk_lut_gen, prec, Es)
    table = soft_dec_table(constel, code, prec)

    c.gen_soft_dec_lut(prec)
    #c.set_soft_dec_lut(table, prec)

    y_python_gen_calc = qpsk_lut_gen(sample, Es)
    y_python_table = calc_soft_dec_from_table(sample, table, prec, Es)
    y_python_raw_calc = calc_soft_dec(sample, constel, code)
    y_cpp_table = c.soft_decision_maker(sample)
    y_cpp_raw_calc = c.calc_soft_dec(sample)

    return (y_python_gen_calc, y_python_table, y_python_raw_calc,
            y_cpp_table, y_cpp_raw_calc, constel, code, c)


def test_qam16(i, sample, prec):
    sample = sample / 1
    qam_const_list = [qam_16_0, ]
    qam_lut_gen_list = [sd_qam_16_0, ]

    constel, code = qam_const_list[i]()
    qam_lut_gen = qam_lut_gen_list[i]

    rot_sym = 4
    side = 2
    width = 2
    c = digital.constellation_rect(constel, code, rot_sym,
                                   side, side, width, width)

    # Get max energy/symbol in constellation
    constel = c.points()
    Es = max([abs(constel_i) for constel_i in constel])

    #table = soft_dec_table_generator(qam_lut_gen, prec, Es)
    table = soft_dec_table(constel, code, prec, 1)

    # c.gen_soft_dec_lut(prec)
    c.set_soft_dec_lut(table, prec)

    y_python_gen_calc = qam_lut_gen(sample, Es)
    y_python_table = calc_soft_dec_from_table(sample, table, prec, Es)
    y_python_raw_calc = calc_soft_dec(sample, constel, code, 1)
    y_cpp_table = c.soft_decision_maker(sample)
    y_cpp_raw_calc = c.calc_soft_dec(sample)

    return (y_python_gen_calc, y_python_table, y_python_raw_calc,
            y_cpp_table, y_cpp_raw_calc, constel, code, c)


if __name__ == "__main__":

    index = 0
    prec = 8

    x_re = 2 * numpy.random.random() - 1
    x_im = 2 * numpy.random.random() - 1
    x = x_re + x_im * 1j
    #x = -1 + -0.j

    if 1:
        y_python_gen_calc, y_python_table, y_python_raw_calc, \
            y_cpp_table, y_cpp_raw_calc, constel, code, c \
            = test_qpsk(index, x, prec)
    else:
        y_python_gen_calc, y_python_table, y_python_raw_calc, \
            y_cpp_table, y_cpp_raw_calc, constel, code, c \
            = test_qam16(index, x, prec)

    k = numpy.log2(len(constel))

    print("Sample: ", x)
    print("Python Generator Calculated: ", (y_python_gen_calc))
    print("Python Generator Table:      ", (y_python_table))
    print("Python Raw calc:             ", (y_python_raw_calc))
    print("C++ Table calc:              ", (y_cpp_table))
    print("C++ Raw calc:                ", (y_cpp_raw_calc))

    fig = pyplot.figure(1)
    sp1 = fig.add_subplot(1, 1, 1)
    sp1.plot([c.real for c in constel],
             [c.imag for c in constel], 'bo')
    sp1.plot(x.real, x.imag, 'ro')
    sp1.set_xlim([-1.5, 1.5])
    sp1.set_ylim([-1.5, 1.5])
    fill = int(numpy.log2(len(constel)))
    for i, c in enumerate(constel):
        sp1.text(1.2 * c.real, 1.2 * c.imag, bin(code[i])[2:].zfill(fill),
                 ha='center', va='center', size=18)
    pyplot.show()



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_multiply_conjugate (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000(self):
        src_data0 = [-2 - 2j, -1 - 1j, -2 + 2j, -1 + 1j,
                     2 - 2j, 1 - 1j, 2 + 2j, 1 + 1j,
                     0 + 0j]
        src_data1 = [-3 - 3j, -4 - 4j, -3 + 3j, -4 + 4j,
                     3 - 3j, 4 - 4j, 3 + 3j, 4 + 4j,
                     0 + 0j]

        exp_data = [12 + 0j, 8 + 0j, 12 + 0j, 8 + 0j,
                    12 + 0j, 8 + 0j, 12 + 0j, 8 + 0j,
                    0 + 0j]
        src0 = blocks.vector_source_c(src_data0)
        src1 = blocks.vector_source_c(src_data1)
        op = blocks.multiply_conjugate_cc()
        dst = blocks.vector_sink_c()

        self.tb.connect(src0, (op, 0))
        self.tb.connect(src1, (op, 1))
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)


if __name__ == '__main__':
    gr_unittest.run(test_multiply_conjugate)



#!/usr/bin/env python
#
# Copyright 2015 Free Software Foundation, Inc.
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from .encoder import PolarEncoder
from .decoder import PolarDecoder
from . import channel_construction as cc
import numpy as np
from .helper_functions import power_of_2_int

import matplotlib.pyplot as plt


def get_frozen_bit_position():
    # frozenbitposition = np.array((0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 12, 16, 17, 18, 20, 24), dtype=int)
    # frozenbitposition = np.array((0, 1, 2, 3, 4, 5, 8, 9), dtype=int)
    m = 256
    n_frozen = m // 2
    frozenbitposition = cc.get_frozen_bit_indices_from_z_parameters(
        cc.bhattacharyya_bounds(0.0, m), n_frozen
    )
    print(frozenbitposition)
    return frozenbitposition


def test_enc_dec_chain():
    ntests = 100
    n = 256
    k = n // 2
    frozenbits = np.zeros(n - k)
    frozenbitposition = get_frozen_bit_position()
    for i in range(ntests):
        bits = np.random.randint(2, size=k)
        encoder = PolarEncoder(n, k, frozenbitposition, frozenbits)
        decoder = PolarDecoder(n, k, frozenbitposition, frozenbits)
        encoded = encoder.encode(bits)
        rx = decoder.decode(encoded)
        if not is_equal(bits, rx):
            raise ValueError(
                "Test #", i, "failed, input and output differ", bits, "!=", rx
            )
            return


def is_equal(first, second):
    if not (first == second).all():
        result = first == second
        for i in range(len(result)):
            print(
                "{0:4}: {1:2} == {2:1} = {3}".format(
                    i, first[i], second[i], result[i])
            )
        return False
    return True


def exact_value(la, lb):
    return np.log((np.exp(la + lb) + 1) / (np.exp(la + np.exp(lb))))


def approx_value(la, lb):
    return np.sign(la) * np.sign(lb) * np.minimum(np.abs(la), np.abs(lb))


def path_metric_exact(last_pm, llr, ui):
    return last_pm + np.log(1 + np.exp(-1.0 * llr * (1 - 2 * ui)))


def path_metric_approx(last_pm, llr, ui):
    if ui == int(0.5 * (1 - np.sign(llr))):
        return last_pm
    return last_pm + np.abs(llr)


def calculate_path_metric_vector(metric, llrs, us):
    res = np.zeros(llrs.size)
    res[0] = metric(0, llrs[0], us[0])
    for i in range(1, llrs.size):
        res[i] = metric(res[i - 1], llrs[i], us[i])
    return res


def test_1024_rate_1_code():
    # effectively a Monte-Carlo simulation for channel polarization.
    ntests = 10000
    n = 256
    k = n
    transition_prob = 0.11
    num_transitions = int(k * transition_prob)
    frozenbits = np.zeros(n - k)
    frozenbitposition = np.array((), dtype=int)
    encoder = PolarEncoder(n, k, frozenbitposition, frozenbits)
    decoder = PolarDecoder(n, k, frozenbitposition, frozenbits)

    channel_counter = np.zeros(k)
    possible_indices = np.arange(n, dtype=int)
    for i in range(ntests):
        bits = np.random.randint(2, size=k)
        tx = encoder.encode(bits)
        np.random.shuffle(possible_indices)
        tx[possible_indices[0:num_transitions]] = (
            tx[possible_indices[0:num_transitions]] + 1
        ) % 2
        rx = tx
        recv = decoder.decode(rx)
        channel_counter += bits == recv

    print(channel_counter)
    print(np.min(channel_counter), np.max(channel_counter))

    np.save("channel_counter_" + str(ntests) + ".npy", channel_counter)


def find_good_indices(res, nindices):
    channel_counter = np.copy(res)
    good_indices = np.zeros(channel_counter.size)

    for i in range(nindices):
        idx = np.argmax(channel_counter)
        good_indices[idx] = 1
        channel_counter[idx] = 0
    return good_indices


def channel_analysis():
    ntests = 10000
    filename = "channel_counter_" + str(ntests) + ".npy"
    channel_counter = np.load(filename)
    print(np.min(channel_counter), np.max(channel_counter))
    channel_counter[0] = np.min(channel_counter)
    good_indices = find_good_indices(
        channel_counter, channel_counter.size // 2)
    info_bit_positions = np.where(good_indices > 0)
    print(info_bit_positions)
    frozen_bit_positions = np.delete(
        np.arange(channel_counter.size), info_bit_positions
    )
    print(frozen_bit_positions)
    np.save("frozen_bit_positions_n256_k128_p0.11.npy", frozen_bit_positions)
    good_indices *= 2000
    good_indices += 4000

    plt.plot(channel_counter)
    plt.plot(good_indices)
    plt.show()


def merge_first_stage(init_mask):
    merged_frozen_mask = []
    for e in range(0, len(init_mask), 2):
        v = [init_mask[e]["value"][0], init_mask[e + 1]["value"][0]]
        s = init_mask[e]["size"] * 2
        if init_mask[e]["type"] == init_mask[e + 1]["type"]:
            t = init_mask[e]["type"]
            merged_frozen_mask.append({"value": v, "type": t, "size": s})
        else:
            t = "RPT"
            merged_frozen_mask.append({"value": v, "type": t, "size": s})
    return merged_frozen_mask


def merge_second_stage(init_mask):
    merged_frozen_mask = []
    for e in range(0, len(init_mask), 2):
        if init_mask[e]["type"] == init_mask[e + 1]["type"]:
            t = init_mask[e]["type"]
            v = init_mask[e]["value"]
            v.extend(init_mask[e + 1]["value"])
            s = init_mask[e]["size"] * 2
            merged_frozen_mask.append({"value": v, "type": t, "size": s})
        elif init_mask[e]["type"] == "ZERO" and init_mask[e + 1]["type"] == "RPT":
            t = init_mask[e + 1]["type"]
            v = init_mask[e]["value"]
            v.extend(init_mask[e + 1]["value"])
            s = init_mask[e]["size"] * 2
            merged_frozen_mask.append({"value": v, "type": t, "size": s})
        elif init_mask[e]["type"] == "RPT" and init_mask[e + 1]["type"] == "ONE":
            t = "SPC"
            v = init_mask[e]["value"]
            v.extend(init_mask[e + 1]["value"])
            s = init_mask[e]["size"] * 2
            merged_frozen_mask.append({"value": v, "type": t, "size": s})
        else:
            merged_frozen_mask.append(init_mask[e])
            merged_frozen_mask.append(init_mask[e + 1])
    return merged_frozen_mask


def merge_stage_n(init_mask):
    merged_frozen_mask = []
    n_elems = len(init_mask) - (len(init_mask) % 2)
    for e in range(0, n_elems, 2):
        if init_mask[e]["size"] == init_mask[e + 1]["size"]:
            if (
                init_mask[e]["type"] == "ZERO" or init_mask[e]["type"] == "ONE"
            ) and init_mask[e]["type"] == init_mask[e + 1]["type"]:
                t = init_mask[e]["type"]
                v = init_mask[e]["value"]
                v.extend(init_mask[e + 1]["value"])
                s = init_mask[e]["size"] * 2
                merged_frozen_mask.append({"value": v, "type": t, "size": s})
            elif init_mask[e]["type"] == "ZERO" and init_mask[e + 1]["type"] == "RPT":
                t = init_mask[e + 1]["type"]
                v = init_mask[e]["value"]
                v.extend(init_mask[e + 1]["value"])
                s = init_mask[e]["size"] * 2
                merged_frozen_mask.append({"value": v, "type": t, "size": s})
            elif init_mask[e]["type"] == "SPC" and init_mask[e + 1]["type"] == "ONE":
                t = init_mask[e]["type"]
                v = init_mask[e]["value"]
                v.extend(init_mask[e + 1]["value"])
                s = init_mask[e]["size"] * 2
                merged_frozen_mask.append({"value": v, "type": t, "size": s})
            else:
                merged_frozen_mask.append(init_mask[e])
                merged_frozen_mask.append(init_mask[e + 1])
        else:
            merged_frozen_mask.append(init_mask[e])
            merged_frozen_mask.append(init_mask[e + 1])
    if n_elems < len(init_mask):
        merged_frozen_mask.append(init_mask[-1])
    return merged_frozen_mask


def print_decode_subframes(subframes):
    for e in subframes:
        print(e)


def find_decoder_subframes(frozen_mask):
    stages = power_of_2_int(len(frozen_mask))
    block_size = 2 ** stages

    lock_mask = np.zeros(block_size, dtype=int)
    sub_mask = []

    for e in frozen_mask:
        if e == 1:
            sub_mask.append(0)
        else:
            sub_mask.append(1)
    sub_mask = np.array(sub_mask, dtype=int)

    for s in range(0, stages):
        stage_size = 2 ** s
        mask = np.reshape(sub_mask, (-1, stage_size))
        lock = np.reshape(lock_mask, (-1, stage_size))
        for p in range(0, (block_size // stage_size) - 1, 2):
            l0 = lock[p]
            l1 = lock[p + 1]
            first = mask[p]
            second = mask[p + 1]
            print(l0, l1)
            print(first, second)
            if np.all(l0 == l1):
                for eq in range(2):
                    if np.all(first == eq) and np.all(second == eq):
                        mask[p].fill(eq)
                        mask[p + 1].fill(eq)
                        lock[p].fill(s)
                        lock[p + 1].fill(s)

                if np.all(first == 0) and np.all(second == 2):
                    mask[p].fill(2)
                    mask[p + 1].fill(2)
                    lock[p].fill(s)
                    lock[p + 1].fill(s)

                if np.all(first == 3) and np.all(second == 1):
                    mask[p].fill(3)
                    mask[p + 1].fill(3)
                    lock[p].fill(s)
                    lock[p + 1].fill(s)

            if s == 0 and np.all(first == 0) and np.all(second == 1):
                mask[p].fill(2)
                mask[p + 1].fill(2)
                lock[p].fill(s)
                lock[p + 1].fill(s)

            if s == 1 and np.all(first == 2) and np.all(second == 1):
                mask[p].fill(3)
                mask[p + 1].fill(3)
                lock[p].fill(s)
                lock[p + 1].fill(s)

        sub_mask = mask.flatten()
        lock_mask = lock.flatten()

    words = {0: "ZERO", 1: "ONE", 2: "RPT", 3: "SPC"}
    ll = lock_mask[0]
    sub_t = sub_mask[0]
    for i in range(len(frozen_mask)):
        v = frozen_mask[i]
        t = words[sub_mask[i]]
        l = lock_mask[i]
        # if i % 8 == 0:
        #     print
        if not l == ll or not sub_mask[i] == sub_t:
            print("--------------------------")
        ll = l
        sub_t = sub_mask[i]
        print("{0:4} lock {1:4} value: {2} in sub {3}".format(
            i, 2 ** (l + 1), v, t))


def systematic_encoder_decoder_chain_test():
    print("systematic encoder decoder chain test")
    block_size = int(2 ** 8)
    info_bit_size = block_size // 2
    ntests = 100
    frozenbitposition = cc.get_frozen_bit_indices_from_z_parameters(
        cc.bhattacharyya_bounds(0.0, block_size), block_size - info_bit_size
    )
    encoder = PolarEncoder(block_size, info_bit_size, frozenbitposition)
    decoder = PolarDecoder(block_size, info_bit_size, frozenbitposition)
    for i in range(ntests):
        bits = np.random.randint(2, size=info_bit_size)
        y = encoder.encode_systematic(bits)
        u_hat = decoder.decode_systematic(y)
        assert (bits == u_hat).all()


def main():
    n = 8
    m = 2 ** n
    k = m // 2
    n_frozen = n - k
    # n = 16
    # k = 8
    # frozenbits = np.zeros(n - k)
    # frozenbitposition8 = np.array((0, 1, 2, 4), dtype=int)
    # frozenbitposition = np.array((0, 1, 2, 3, 4, 5, 8, 9), dtype=int)
    # print(frozenbitposition)

    # test_enc_dec_chain()
    # test_1024_rate_1_code()
    # channel_analysis()

    # frozen_indices = cc.get_bec_frozen_indices(m, n_frozen, 0.11)
    # frozen_mask = cc.get_frozen_bit_mask(frozen_indices, m)
    # find_decoder_subframes(frozen_mask)

    systematic_encoder_decoder_chain_test()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# Copyright 2015 Free Software Foundation, Inc.
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import numpy as np
from .common import PolarCommon

# for dev
from .encoder import PolarEncoder
from matplotlib import pyplot as plt


class PolarDecoder(PolarCommon):
    def __init__(self, n, k, frozen_bit_position, frozenbits=None):
        PolarCommon.__init__(self, n, k, frozen_bit_position, frozenbits)

        # this is kind of a dummy value. usually chosen individually.
        self.error_probability = 0.1
        self.lrs = ((1 - self.error_probability) / self.error_probability,
                    self.error_probability / (1 - self.error_probability))
        self.llrs = np.log(self.lrs)

    def _llr_bit(self, bit):
        return self.llrs[bit]

    def _llr_odd(self, la, lb):
        # this functions uses the min-sum approximation
        # exact formula: np.log((np.exp(la + lb) + 1) / (np.exp(la) + np.exp(lb)))
        return np.sign(la) * np.sign(lb) * np.minimum(np.abs(la), np.abs(lb))

    _f_vals = np.array((1.0, -1.0), dtype=float)

    def _llr_even(self, la, lb, f):
        return (la * self._f_vals[f]) + lb

    def _llr_bit_decision(self, llr):
        if llr < 0.0:
            ui = int(1)
        else:
            ui = int(0)
        return ui

    def _retrieve_bit_from_llr(self, lr, pos):
        f_index = np.where(self.frozen_bit_position == pos)[0]
        if not f_index.size == 0:
            ui = self.frozenbits[f_index][0]
        else:
            ui = self._llr_bit_decision(lr)
        return ui

    def _lr_bit(self, bit):
        return self.lrs[bit]

    def _lr_odd(self, la, lb):
        # la is upper branch and lb is lower branch
        return (la * lb + 1) / (la + lb)

    def _lr_even(self, la, lb, f):
        # la is upper branch and lb is lower branch, f is last decoded bit.
        return (la ** (1 - (2 * f))) * lb

    def _lr_bit_decision(self, lr):
        if lr < 1:
            return int(1)
        return int(0)

    def _get_even_indices_values(self, u_hat):
        # looks like overkill for some indexing, but zero and one based indexing mix-up gives you haedaches.
        return u_hat[1::2]

    def _get_odd_indices_values(self, u_hat):
        return u_hat[0::2]

    def _calculate_lrs(self, y, u):
        ue = self._get_even_indices_values(u)
        uo = self._get_odd_indices_values(u)
        ya = y[0:y.size // 2]
        yb = y[(y.size // 2):]
        la = self._lr_decision_element(ya, (ue + uo) % 2)
        lb = self._lr_decision_element(yb, ue)
        return la, lb

    def _lr_decision_element(self, y, u):
        if y.size == 1:
            return self._llr_bit(y[0])
        if u.size % 2 == 0:  # use odd branch formula
            la, lb = self._calculate_lrs(y, u)
            return self._llr_odd(la, lb)
        else:
            ui = u[-1]
            la, lb = self._calculate_lrs(y, u[0:-1])
            return self._llr_even(la, lb, ui)

    def _retrieve_bit_from_lr(self, lr, pos):
        f_index = np.where(self.frozen_bit_position == pos)[0]
        if not f_index.size == 0:
            ui = self.frozenbits[f_index][0]
        else:
            ui = self._lr_bit_decision(lr)
        return ui

    def _lr_sc_decoder(self, y):
        # this is the standard SC decoder as derived from the formulas. It sticks to natural bit order.
        u = np.array([], dtype=int)
        for i in range(y.size):
            lr = self._lr_decision_element(y, u)
            ui = self._retrieve_bit_from_llr(lr, i)
            u = np.append(u, ui)
        return u

    def _llr_retrieve_bit(self, llr, pos):
        f_index = np.where(self.frozen_bit_position == pos)[0]
        if not f_index.size == 0:
            ui = self.frozenbits[f_index][0]
        else:
            ui = self._llr_bit_decision(llr)
        return ui

    def _butterfly_decode_bits(self, pos, graph, u):
        bit_num = u.size
        llr = graph[pos][0]
        ui = self._llr_retrieve_bit(llr, bit_num)
        # ui = self._llr_bit_decision(llr)
        u = np.append(u, ui)
        lower_right = pos + (self.N // 2)
        la = graph[pos][1]
        lb = graph[lower_right][1]
        graph[lower_right][0] = self._llr_even(la, lb, ui)
        llr = graph[lower_right][0]
        # ui = self._llr_bit_decision(llr)
        ui = self._llr_retrieve_bit(llr, u.size)
        u = np.append(u, ui)
        return graph, u

    def _lr_sc_decoder_efficient(self, y):
        graph = np.full((self.N, self.power + 1), np.NaN, dtype=float)
        for i in range(self.N):
            graph[i][self.power] = self._llr_bit(y[i])
        decode_order = self._vector_bit_reversed(np.arange(self.N), self.power)
        decode_order = np.delete(
            decode_order, np.where(decode_order >= self.N // 2))
        u = np.array([], dtype=int)
        for pos in decode_order:
            graph = self._butterfly(pos, 0, graph, u)
            graph, u = self._butterfly_decode_bits(pos, graph, u)
        return u

    def _stop_propagation(self, bf_entry_row, stage):
        # calculate break condition
        modulus = 2 ** (self.power - stage)
        # stage_size = self.N // (2 ** stage)
        # half_stage_size = stage_size // 2
        half_stage_size = self.N // (2 ** (stage + 1))
        stage_pos = bf_entry_row % modulus
        return stage_pos >= half_stage_size

    def _butterfly(self, bf_entry_row, stage, graph, u):
        if not self.power > stage:
            return graph

        if self._stop_propagation(bf_entry_row, stage):
            upper_right = bf_entry_row - self.N // (2 ** (stage + 1))
            la = graph[upper_right][stage + 1]
            lb = graph[bf_entry_row][stage + 1]
            ui = u[-1]
            graph[bf_entry_row][stage] = self._llr_even(la, lb, ui)
            return graph

        # activate right side butterflies
        u_even = self._get_even_indices_values(u)
        u_odd = self._get_odd_indices_values(u)
        graph = self._butterfly(bf_entry_row, stage + 1,
                                graph, (u_even + u_odd) % 2)
        lower_right = bf_entry_row + self.N // (2 ** (stage + 1))
        graph = self._butterfly(lower_right, stage + 1, graph, u_even)

        la = graph[bf_entry_row][stage + 1]
        lb = graph[lower_right][stage + 1]
        graph[bf_entry_row][stage] = self._llr_odd(la, lb)
        return graph

    def decode(self, data, is_packed=False):
        if not len(data) == self.N:
            raise ValueError(
                "len(data)={0} is not equal to n={1}!".format(len(data), self.N))
        if is_packed:
            data = np.unpackbits(data)
        data = self._lr_sc_decoder_efficient(data)
        data = self._extract_info_bits(data)
        if is_packed:
            data = np.packbits(data)
        return data

    def _extract_info_bits_reversed(self, y):
        info_bit_positions_reversed = self._vector_bit_reversed(
            self.info_bit_position, self.power)
        return y[info_bit_positions_reversed]

    def decode_systematic(self, data):
        if not len(data) == self.N:
            raise ValueError(
                "len(data)={0} is not equal to n={1}!".format(len(data), self.N))
        # data = self._reverse_bits(data)
        data = self._lr_sc_decoder_efficient(data)
        data = self._encode_natural_order(data)
        data = self._extract_info_bits_reversed(data)
        return data


def test_systematic_decoder():
    ntests = 1000
    n = 16
    k = 8
    frozenbitposition = np.array((0, 1, 2, 3, 4, 5, 8, 9), dtype=int)
    encoder = PolarEncoder(n, k, frozenbitposition)
    decoder = PolarDecoder(n, k, frozenbitposition)
    for i in range(ntests):
        bits = np.random.randint(2, size=k)
        y = encoder.encode_systematic(bits)
        u_hat = decoder.decode_systematic(y)
        assert (bits == u_hat).all()


def test_reverse_enc_dec():
    n = 16
    k = 8
    frozenbits = np.zeros(n - k)
    frozenbitposition = np.array((0, 1, 2, 3, 4, 5, 8, 9), dtype=int)
    bits = np.random.randint(2, size=k)
    encoder = PolarEncoder(n, k, frozenbitposition, frozenbits)
    decoder = PolarDecoder(n, k, frozenbitposition, frozenbits)
    encoded = encoder.encode(bits)
    print('encoded:', encoded)
    rx = decoder.decode(encoded)
    print('bits:', bits)
    print('rx  :', rx)
    print((bits == rx).all())


def compare_decoder_impls():
    print('\nthis is decoder test')
    n = 8
    k = 4
    frozenbits = np.zeros(n - k)
    # frozenbitposition16 = np.array((0, 1, 2, 3, 4, 5, 8, 9), dtype=int)
    frozenbitposition = np.array((0, 1, 2, 4), dtype=int)
    bits = np.random.randint(2, size=k)
    print('bits:', bits)
    encoder = PolarEncoder(n, k, frozenbitposition, frozenbits)
    decoder = PolarDecoder(n, k, frozenbitposition, frozenbits)
    encoded = encoder.encode(bits)
    print('encoded:', encoded)
    rx_st = decoder._lr_sc_decoder(encoded)
    rx_eff = decoder._lr_sc_decoder_efficient(encoded)
    print('standard :', rx_st)
    print('efficient:', rx_eff)
    print((rx_st == rx_eff).all())


def main():
    # power = 3
    # n = 2 ** power
    # k = 4
    # frozenbits = np.zeros(n - k, dtype=int)
    # frozenbitposition = np.array((0, 1, 2, 4), dtype=int)
    # frozenbitposition4 = np.array((0, 1), dtype=int)
    #
    #
    # encoder = PolarEncoder(n, k, frozenbitposition, frozenbits)
    # decoder = PolarDecoder(n, k, frozenbitposition, frozenbits)
    #
    # bits = np.ones(k, dtype=int)
    # print("bits: ", bits)
    # evec = encoder.encode(bits)
    # print("froz: ", encoder._insert_frozen_bits(bits))
    # print("evec: ", evec)
    #
    # evec[1] = 0
    # deced = decoder._lr_sc_decoder(evec)
    # print('SC decoded:', deced)
    #
    # test_reverse_enc_dec()
    # compare_decoder_impls()

    test_systematic_decoder()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2017 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest
from gnuradio import blocks
import pmt


class qa_exponentiate_const_cci(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_t(self):
        for exponent in range(1, 10):
            in_data = [1 + 1j, -1, 4 - 1j, -3 - 7j]
            out_data = [
                in_data[0]**exponent,
                in_data[1]**exponent,
                in_data[2]**exponent,
                in_data[3]**exponent]

            # Test streaming input
            source = blocks.vector_source_c(in_data, False, 1)
            exponentiate_const_cci = blocks.exponentiate_const_cci(exponent)
            sink = blocks.vector_sink_c(1)

            self.tb.connect(source, exponentiate_const_cci, sink)
            self.tb.run()

            self.assertAlmostEqual(sink.data(), out_data)

            # Test vector input
            for vlen in [2, 4]:
                source = blocks.vector_source_c(in_data, False, 1)
                s2v = blocks.stream_to_vector(gr.sizeof_gr_complex, vlen)
                exponentiate_const_cci = blocks.exponentiate_const_cci(
                    exponent, vlen)
                v2s = blocks.vector_to_stream(gr.sizeof_gr_complex, vlen)
                sink = blocks.vector_sink_c(1)

                self.tb.connect(source, s2v, exponentiate_const_cci, v2s, sink)
                self.tb.run()

                self.assertAlmostEqual(sink.data(), out_data)


if __name__ == '__main__':
    gr_unittest.run(qa_exponentiate_const_cci)



#!/usr/bin/env python
#
# Copyright 2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import math
import numpy as np
from gnuradio import gr, gr_unittest, digital
from gnuradio.digital.utils import lfsr_args


class test_lfsr(gr_unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_lfsr_001(self):
        reglen = 8
        l = digital.lfsr(1, 1, reglen)
        result_data = []
        for i in range(4 * (reglen + 1)):
            result_data.append(l.next_bit())

        expected_result = 4 * ([1, ] + reglen * [0, ])
        self.assertFloatTuplesAlmostEqual(expected_result, result_data, 5)

    def test_lfsr_002(self):
        l = digital.lfsr(*lfsr_args(0b1, 5, 3, 0))
        result_data = [l.next_bit() for _ in range(2 * (2**5 - 1))]

        expected_result = [1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0,
                           0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0] * 2
        self.assertEqual(expected_result, result_data)

        seq1 = [l.next_bit() for _ in range(2**5 - 1)]
        seq2 = [l.next_bit() for _ in range(2**5 - 1)]
        self.assertEqual(seq1, seq2)

        res = (np.convolve(seq1, [1, 0, 1, 0, 0, 1]) % 2)
        self.assertEqual(sum(res[5:-5]), 0, msg="LRS not generated properly")


if __name__ == '__main__':
    gr_unittest.run(test_lfsr)



#!/usr/bin/env python
#
# Copyright 2004,2007,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, filter, blocks


class test_iir_filter(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_iir_direct_001(self):
        src_data = (1, 2, 3, 4, 5, 6, 7, 8)
        fftaps = ()
        fbtaps = ()
        expected_result = (0, 0, 0, 0, 0, 0, 0, 0)
        src = blocks.vector_source_f(src_data)
        op = filter.iir_filter_ffd(fftaps, fbtaps)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_iir_direct_002(self):
        src_data = (1, 2, 3, 4, 5, 6, 7, 8)
        fftaps = (2,)
        fbtaps = (0,)
        expected_result = (2, 4, 6, 8, 10, 12, 14, 16)
        src = blocks.vector_source_f(src_data)
        op = filter.iir_filter_ffd(fftaps, fbtaps)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_iir_direct_003(self):
        src_data = (1, 2, 3, 4, 5, 6, 7, 8)
        fftaps = (2, 11)
        fbtaps = (0, 0)
        expected_result = (2, 15, 28, 41, 54, 67, 80, 93)
        src = blocks.vector_source_f(src_data)
        op = filter.iir_filter_ffd(fftaps, fbtaps)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_iir_direct_004(self):
        src_data = (1, 2, 3, 4, 5, 6, 7, 8)
        fftaps = (2, 11)
        fbtaps = (0, -1)
        expected_result = (2, 13, 15, 26, 28, 39, 41, 52)
        src = blocks.vector_source_f(src_data)
        op = filter.iir_filter_ffd(fftaps, fbtaps)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_iir_direct_005(self):
        src_data = (1, 2, 3, 4, 5, 6, 7, 8)
        fftaps = (2, 11, 0)
        fbtaps = (0, -1, 3)
        expected_result = (2, 13, 21, 59, 58, 186, 68, 583)
        src = blocks.vector_source_f(src_data)
        op = filter.iir_filter_ffd(fftaps, fbtaps)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_iir_direct_006(self):
        src_data = (1, 2, 3, 4, 5, 6, 7, 8)
        expected_result = (2, 13, 21, 59, 58, 186, 68, 583)
        fftaps = (2, 1)
        fbtaps = (0, -1)
        src = blocks.vector_source_f(src_data)
        op = filter.iir_filter_ffd(fftaps, fbtaps)
        fftaps = (2, 11, 0)
        fbtaps = (0, -1, 3)
        op.set_taps(fftaps, fbtaps)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_iir_direct_007(self):
        src_data = (1, 2, 3, 4, 5, 6, 7, 8)
        expected_result = (2, 2, 5, 5, 8, 8, 11, 11)
        fftaps = (2, 1)
        fbtaps = (0, -1)
        src = blocks.vector_source_f(src_data)
        op = filter.iir_filter_ffd(fftaps, fbtaps)
        fftaps = (2, 0, 1)
        fbtaps = (0, -1)
        op.set_taps(fftaps, fbtaps)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_iir_direct_008(self):
        src_data = (1, 2, 3, 4, 5, 6, 7, 8)
        expected_result = (2, 4, 4, 10, 18, 14, 26, 56)
        fftaps = (2,)
        fbtaps = (0, 1)
        src = blocks.vector_source_f(src_data)
        op = filter.iir_filter_ffd(fftaps, fbtaps)
        fftaps_data = (1)
        fbtaps = (0, 0, -1, 3)
        op.set_taps(fftaps, fbtaps)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_iir_ccf_001(self):
        src_data = (
            1 + 1j,
            2 + 2j,
            3 + 3j,
            4 + 4j,
            5 + 5j,
            6 + 6j,
            7 + 7j,
            8 + 8j)
        expected_result = (
            2 + 2j,
            (6 + 6j),
            (12 + 12j),
            (20 + 20j),
            (30 + 30j),
            (42 + 42j),
            (56 + 56j),
            (72 + 72j))
        fftaps = (2,)
        fbtaps = (0, 1)
        src = blocks.vector_source_c(src_data)
        op = filter.iir_filter_ccf(fftaps, fbtaps)
        dst = blocks.vector_sink_c()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_iir_ccf_002(self):
        src_data = (
            1 + 1j,
            2 + 2j,
            3 + 3j,
            4 + 4j,
            5 + 5j,
            6 + 6j,
            7 + 7j,
            8 + 8j)
        expected_result = (
            2 + 2j,
            (6 + 6j),
            (12 + 12j),
            (20 + 20j),
            (30 + 30j),
            (42 + 42j),
            (56 + 56j),
            (72 + 72j))

        src = blocks.vector_source_c(src_data)
        op = filter.iir_filter_ccf([1], [1])
        dst = blocks.vector_sink_c()

        fftaps = (2,)
        fbtaps = (0, 1)
        op.set_taps(fftaps, fbtaps)

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_iir_ccd_001(self):
        src_data = (
            1 + 1j,
            2 + 2j,
            3 + 3j,
            4 + 4j,
            5 + 5j,
            6 + 6j,
            7 + 7j,
            8 + 8j)
        expected_result = (
            2 + 2j,
            (6 + 6j),
            (12 + 12j),
            (20 + 20j),
            (30 + 30j),
            (42 + 42j),
            (56 + 56j),
            (72 + 72j))
        fftaps = (2,)
        fbtaps = (0, 1)
        src = blocks.vector_source_c(src_data)
        op = filter.iir_filter_ccd(fftaps, fbtaps)
        dst = blocks.vector_sink_c()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_iir_ccd_002(self):
        src_data = (
            1 + 1j,
            2 + 2j,
            3 + 3j,
            4 + 4j,
            5 + 5j,
            6 + 6j,
            7 + 7j,
            8 + 8j)
        expected_result = (
            2 + 2j,
            (6 + 6j),
            (12 + 12j),
            (20 + 20j),
            (30 + 30j),
            (42 + 42j),
            (56 + 56j),
            (72 + 72j))

        src = blocks.vector_source_c(src_data)
        op = filter.iir_filter_ccd([1], [1])
        dst = blocks.vector_sink_c()

        fftaps = (2,)
        fbtaps = (0, 1)
        op.set_taps(fftaps, fbtaps)

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_iir_ccc_001(self):
        src_data = (
            1 + 1j,
            2 + 2j,
            3 + 3j,
            4 + 4j,
            5 + 5j,
            6 + 6j,
            7 + 7j,
            8 + 8j)
        expected_result = (4j, 12j, 24j, 40j, 60j, 84j, 112j, 144j)
        fftaps = (2 + 2j,)
        fbtaps = (0, 1)
        src = blocks.vector_source_c(src_data)
        op = filter.iir_filter_ccc(fftaps, fbtaps)
        dst = blocks.vector_sink_c()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_iir_ccc_002(self):
        src_data = (
            1 + 1j,
            2 + 2j,
            3 + 3j,
            4 + 4j,
            5 + 5j,
            6 + 6j,
            7 + 7j,
            8 + 8j)
        expected_result = (4j, 12j, 24j, 40j, 60j, 84j, 112j, 144j)

        src = blocks.vector_source_c(src_data)
        op = filter.iir_filter_ccc([1], [1])
        dst = blocks.vector_sink_c()

        fftaps = (2 + 2j,)
        fbtaps = (0, 1)
        op.set_taps(fftaps, fbtaps)

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_iir_ccz_001(self):
        src_data = (
            1 + 1j,
            2 + 2j,
            3 + 3j,
            4 + 4j,
            5 + 5j,
            6 + 6j,
            7 + 7j,
            8 + 8j)
        expected_result = (4j, 12j, 24j, 40j, 60j, 84j, 112j, 144j)
        fftaps = (2 + 2j,)
        fbtaps = (0, 1)
        src = blocks.vector_source_c(src_data)
        op = filter.iir_filter_ccz(fftaps, fbtaps)
        dst = blocks.vector_sink_c()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_iir_ccz_002(self):
        src_data = (
            1 + 1j,
            2 + 2j,
            3 + 3j,
            4 + 4j,
            5 + 5j,
            6 + 6j,
            7 + 7j,
            8 + 8j)
        expected_result = (4j, 12j, 24j, 40j, 60j, 84j, 112j, 144j)

        src = blocks.vector_source_c(src_data)
        op = filter.iir_filter_ccz([1], [1])
        dst = blocks.vector_sink_c()

        fftaps = (2 + 2j,)
        fbtaps = (0, 1)
        op.set_taps(fftaps, fbtaps)

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)


if __name__ == '__main__':
    gr_unittest.run(test_iir_filter)



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
# Copyright 2021 Marcus Müller
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks, pdu
import pmt
import time


class test_pdu(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_just_run_data(self):
        # Just run some data through and make sure it doesn't puke.
        src = pdu.pdu_to_tagged_stream(gr.types.byte_t)
        snk3 = pdu.tagged_stream_to_pdu(gr.types.byte_t)
        snk2 = blocks.vector_sink_b()
        snk = blocks.tag_debug(1, "test")
        snk.set_display(False)

        dbg = blocks.message_debug()

        # Test that the right number of ports exist.
        pi = snk3.message_ports_in()
        po = snk3.message_ports_out()
        # system port is defined automatically
        self.assertEqual(pmt.length(pi), 1)
        self.assertEqual(pmt.length(po), 1)

        self.tb.connect(src, snk)
        self.tb.connect(src, snk2)
        self.tb.connect(src, snk3)
        self.tb.msg_connect(snk3, "pdus", dbg, "store")

        # make our reference and message pmts
        port = pmt.intern("pdus")
        msg = pmt.cons(pmt.PMT_NIL, pmt.make_u8vector(16, 0xFF))

        # post the message
        src.to_basic_block()._post(port, msg)
        src.to_basic_block()._post(
            pmt.intern("system"), pmt.cons(
                pmt.intern("done"), pmt.from_long(1)))

        self.tb.start()
        self.tb.wait()

        # Get the vector of data from the vector sink
        result_data = snk2.data()

        # Get the vector of data from the message sink
        # Convert the message PMT as a pair into its vector
        result_msg = dbg.get_message(0)
        msg_vec = pmt.cdr(result_msg)
        # pmt.print(msg_vec)

        # Convert the PMT vector into a Python list
        msg_data = []
        for i in range(16):
            msg_data.append(pmt.u8vector_ref(msg_vec, i))

        actual_data = 16 * [0xFF, ]
        self.assertEqual(actual_data, list(result_data))
        self.assertEqual(actual_data, msg_data)

    def test_002_overflow(self):
        # Test the overflow buffer in pdu_to_tagged_stream
        src_data = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0]
        src = pdu.pdu_to_tagged_stream(gr.types.float_t)
        snk = blocks.vector_sink_f()

        self.tb.connect(src, snk)
        port = pmt.intern("pdus")

        msg = pmt.cons(pmt.PMT_NIL, pmt.init_f32vector(10, src_data))
        src.to_basic_block()._post(port, msg)
        src.to_basic_block()._post(
            pmt.intern("system"), pmt.cons(
                pmt.intern("done"), pmt.from_long(1)))

        self.tb.start()
        self.tb.wait()

        self.assertEqual(src_data, list(snk.data()))

    def test_003_tags_plus_data(self):
        packet_len = 16
        src_data = list(range(packet_len))
        tag1 = gr.tag_t()
        tag1.offset = 0
        tag1.key = pmt.string_to_symbol('spam')
        tag1.value = pmt.from_long(23)
        tag2 = gr.tag_t()
        tag2.offset = 10  # Must be < packet_len
        tag2.key = pmt.string_to_symbol('eggs')
        tag2.value = pmt.from_long(42)
        src = blocks.vector_source_f(src_data, tags=(tag1, tag2))
        s2ts = blocks.stream_to_tagged_stream(
            gr.sizeof_float,
            vlen=1,
            packet_len=packet_len,
            len_tag_key="packet_len")
        ts2pdu = pdu.tagged_stream_to_pdu(gr.types.float_t, "packet_len")
        dbg = blocks.message_debug()
        self.tb.connect(src, s2ts, ts2pdu)
        self.tb.msg_connect(ts2pdu, "pdus", dbg, "store")
        self.tb.start()
        self.tb.wait()
        result_msg = dbg.get_message(0)
        metadata = pmt.to_python(pmt.car(result_msg))
        vector = pmt.f32vector_elements(pmt.cdr(result_msg))
        self.assertEqual(metadata, {'eggs': 42, 'spam': 23})
        self.assertFloatTuplesAlmostEqual(tuple(vector), src_data)

    def test_004_legacy_pdu_handling(self):
        self.tb.start()
        self.tb.lock()
        rem = pdu.pdu_remove(pmt.intern('foo'))
        dbg = blocks.message_debug()
        self.tb.msg_connect((rem, 'pdus'), (dbg, 'store'))

        self.tb.unlock()

        msg = pmt.cons(pmt.PMT_NIL, pmt.init_u8vector(3, (1, 2, 3)))
        rem.to_basic_block()._post(pmt.intern('pdus'), msg)
        time.sleep(0.2)

        self.tb.stop()

        self.assertEqual(dbg.num_messages(), 1)
        data = pmt.u8vector_elements(pmt.cdr(dbg.get_message(0)))
        self.assertEqual([1, 2, 3], data)


if __name__ == '__main__':
    gr_unittest.run(test_pdu)



#!/usr/bin/env python
#
# Copyright 2006,2007,2010,2011,2013,2016 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, digital, blocks
import pmt
import numpy
import struct

default_access_code = '\xAC\xDD\xA4\xE2\xF2\x8C\x20\xFC'


def string_to_1_0_list(s):
    r = []
    for ch in s:
        x = ord(ch)
        for i in range(8):
            t = (x >> i) & 0x1
            r.append(t)
    return r


def to_1_0_string(L):
    return ''.join([chr(x + ord('0')) for x in L])


class test_correlate_access_code_XX_ts(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        payload = "test packet"     # payload length is 11 bytes
        # header contains packet length, twice (bit-swapped)
        header = "\x00\xd0\x00\xd0"
        packet = header + payload
        pad = (0,) * 64
        src_data = (0, 0, 1, 1, 1, 1, 0, 1, 1) + \
            tuple(string_to_1_0_list(packet)) + pad
        expected = list(map(int, src_data[9 + 32:-len(pad)]))
        src = blocks.vector_source_b(src_data)
        op = digital.correlate_access_code_bb_ts("1011", 0, "sync")
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        result_tags = dst.tags()
        self.assertEqual(len(result_data), len(payload) * 8)
        self.assertEqual(result_tags[0].offset, 0)
        self.assertEqual(pmt.to_long(result_tags[0].value), len(payload) * 8)
        self.assertEqual(result_data, expected)

    def _variable_payload_t(self, payload_len=2**15 + 3):
        # Constants for this test
        SEED = 42
        PRECEDING_NOISE_LEN = 2**10 + 1
        PREAMBLE_LEN = 31
        PADDING_LEN = 64

        numpy.random.seed(SEED)
        noise_pre = numpy.random.randint(0, 2, size=PRECEDING_NOISE_LEN, dtype=numpy.uint8)
        access_code = numpy.random.randint(0, 2, size=PREAMBLE_LEN, dtype=numpy.uint8)
        payload = numpy.random.randint(0, 256, size=payload_len, dtype=numpy.uint8)

        # header contains packet length, twice (bit-swapped)
        header = numpy.array([(payload_len & 0xFF00) >> 8, payload_len & 0xFF] * 2, dtype=numpy.uint8)
        # make sure we've built the length header correctly
        length1, length2 = struct.unpack(">HH", header)
        self.assertEqual(length1, length2)
        self.assertEqual(length1, len(payload))

        packet = numpy.concatenate((header, payload))
        pad = (0,) * PADDING_LEN
        src_data = numpy.concatenate((
            noise_pre,
            access_code,
            numpy.unpackbits(packet),
            numpy.array(pad, dtype=numpy.uint8)
        ))

        expected = list(src_data[len(noise_pre) + len(access_code) + 32:-len(pad)])
        src = blocks.vector_source_b(src_data)
        op = digital.correlate_access_code_bb_ts("".join((str(b) for b in access_code)), 0, "sync")
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        result_tags = dst.tags()
        self.assertEqual(len(result_data), len(payload) * 8)
        self.assertEqual(result_tags[0].offset, 0)
        self.assertEqual(pmt.to_long(result_tags[0].value), len(payload) * 8)
        self.assertEqual(result_data, expected)

    def test_payload_lengths(self):
        for length in (2**15 + 3, 1, 10, 100, 300, 301, 400, 2**12, 2**16 - 1):
            self._variable_payload_t(length)

    def test_bb_prefix(self):
        payload = "test packet"     # payload length is 11 bytes
        # header contains packet length, twice (bit-swapped)
        header = "\x00\xd0\x00\xd0"
        packet = header + payload
        pad = (0,) * 64
        src_data = (0, 1, 1, 1, 0, 0, 0, 1, 1) + \
            tuple(string_to_1_0_list(packet)) + pad
        expected = list(map(int, src_data[9 + 32:-len(pad)]))
        src = blocks.vector_source_b(src_data)
        op = digital.correlate_access_code_bb_ts("0011", 0, "sync")
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        result_tags = dst.tags()
        self.assertEqual(len(result_data), len(payload) * 8)
        self.assertEqual(result_tags[0].offset, 0)
        self.assertEqual(pmt.to_long(result_tags[0].value), len(payload) * 8)
        self.assertEqual(result_data, expected)

    def test_bb_immediate(self):
        """Test that packets at start of stream match"""
        payload = "test packet"     # payload length is 11 bytes
        # header contains packet length, twice (bit-swapped)
        header = "\x00\xd0\x00\xd0"
        packet = header + payload
        pad = (0,) * 64
        src_data = (0, 0, 1, 1) + tuple(string_to_1_0_list(packet)) + pad
        expected = list(map(int, src_data[4 + 32:-len(pad)]))
        src = blocks.vector_source_b(src_data)
        op = digital.correlate_access_code_bb_ts("0011", 0, "sync")
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        result_tags = dst.tags()
        # self.assertEqual(len(result_data), len(packet)*8)
        self.assertEqual(result_tags[0].offset, 0)
        # self.assertEqual(pmt.to_long(result_tags[0].value), len(payload)*8)
        self.assertEqual(result_data, expected)

    def test_002(self):
        payload = "test packet"     # payload length is 11 bytes
        # header contains packet length, twice (bit-swapped)
        header = "\x00\xd0\x00\xd0"
        packet = header + payload
        pad = (0,) * 64
        src_data = (0, 0, 1, 1, 1, 1, 0, 1, 1) + \
            tuple(string_to_1_0_list(packet)) + pad
        # convert to binary antipodal symbols (-1,1)
        src_floats = tuple(2 * b - 1 for b in src_data)
        expected = src_floats[9 + 32:-len(pad)]
        src = blocks.vector_source_f(src_floats)
        op = digital.correlate_access_code_ff_ts("1011", 0, "sync")
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        result_tags = dst.tags()
        self.assertEqual(len(result_data), len(payload) * 8)
        self.assertEqual(result_tags[0].offset, 0)
        self.assertEqual(pmt.to_long(result_tags[0].value), len(payload) * 8)
        self.assertFloatTuplesAlmostEqual(result_data, expected, 5)

    def test_ff_prefix(self):
        payload = "test packet"     # payload length is 11 bytes
        # header contains packet length, twice (bit-swapped)
        header = "\x00\xd0\x00\xd0"
        packet = header + payload
        pad = (0,) * 64
        src_data = (0, 1, 1, 1, 1, 0, 0, 1, 1) + \
            tuple(string_to_1_0_list(packet)) + pad
        # convert to binary antipodal symbols (-1,1)
        src_floats = tuple(2 * b - 1 for b in src_data)
        expected = src_floats[9 + 32:-len(pad)]
        src = blocks.vector_source_f(src_floats)
        op = digital.correlate_access_code_ff_ts("0011", 0, "sync")
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        result_tags = dst.tags()
        self.assertEqual(len(result_data), len(payload) * 8)
        self.assertEqual(result_tags[0].offset, 0)
        self.assertEqual(pmt.to_long(result_tags[0].value), len(payload) * 8)
        self.assertFloatTuplesAlmostEqual(result_data, expected, 5)

    def test_ff_immediate(self):
        """Test that packets at start of stream match"""
        payload = "test packet"     # payload length is 11 bytes
        # header contains packet length, twice (bit-swapped)
        header = "\x00\xd0\x00\xd0"
        packet = header + payload
        pad = (0,) * 64
        src_data = (0, 0, 1, 1) + tuple(string_to_1_0_list(packet)) + pad
        # convert to binary antipodal symbols (-1,1)
        src_floats = tuple(2 * b - 1 for b in src_data)
        expected = src_floats[4 + 32:-len(pad)]
        src = blocks.vector_source_f(src_floats)
        op = digital.correlate_access_code_ff_ts("0011", 0, "sync")
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = dst.data()
        result_tags = dst.tags()
        self.assertEqual(len(result_data), len(payload) * 8)
        self.assertEqual(result_tags[0].offset, 0)
        self.assertEqual(pmt.to_long(result_tags[0].value), len(payload) * 8)
        self.assertFloatTuplesAlmostEqual(result_data, expected, 5)


if __name__ == '__main__':
    gr_unittest.run(test_correlate_access_code_XX_ts)



#!/usr/bin/env python
#
# Copyright 2009,2011 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr
from gnuradio import filter, fft
from gnuradio import blocks
from gnuradio import uhd
from gnuradio import eng_notation
from gnuradio.eng_option import eng_option
from optparse import OptionParser
import sys

try:
    from gnuradio import qtgui
    from PyQt5 import QtGui, QtCore
    import sip
except ImportError:
    print("Error: Program requires PyQt5 and gr-qtgui.")
    sys.exit(1)

try:
    from usrp_display_qtgui import Ui_MainWindow
except ImportError:
    print("Error: could not find usrp_display_qtgui.py:")
    print("\t\"pyuic4 usrp_display_qtgui.ui -o usrp_display_qtgui.py\"")
    sys.exit(1)


# ////////////////////////////////////////////////////////////////////
#        Define the QT Interface and Control Dialog
# ////////////////////////////////////////////////////////////////////


class main_window(QtGui.QMainWindow):
    def __init__(self, snk, fg, parent=None):

        QtGui.QWidget.__init__(self, parent)
        self.gui = Ui_MainWindow()
        self.gui.setupUi(self)

        self.fg = fg

        # Add the qtsnk widgets to the layout box
        self.gui.sinkLayout.addWidget(snk)

        self.gui.dcGainEdit.setText(QtCore.QString("%1").arg(0.001))

        # Connect up some signals
        self.connect(self.gui.pauseButton, QtCore.SIGNAL("clicked()"),
                     self.pauseFg)
        self.connect(self.gui.frequencyEdit, QtCore.SIGNAL("editingFinished()"),
                     self.frequencyEditText)
        self.connect(self.gui.gainEdit, QtCore.SIGNAL("editingFinished()"),
                     self.gainEditText)
        self.connect(self.gui.bandwidthEdit, QtCore.SIGNAL("editingFinished()"),
                     self.bandwidthEditText)
        self.connect(self.gui.amplifierEdit, QtCore.SIGNAL("editingFinished()"),
                     self.amplifierEditText)

        self.connect(self.gui.actionSaveData, QtCore.SIGNAL("activated()"),
                     self.saveData)
        self.gui.actionSaveData.setShortcut(QtGui.QKeySequence.Save)

        self.connect(self.gui.dcGainEdit, QtCore.SIGNAL("editingFinished()"),
                     self.dcGainEditText)
        self.connect(self.gui.dcCancelCheckBox, QtCore.SIGNAL("clicked(bool)"),
                     self.dcCancelClicked)

    def pauseFg(self):
        if(self.gui.pauseButton.text() == "Pause"):
            self.fg.stop()
            self.fg.wait()
            self.gui.pauseButton.setText("Unpause")
        else:
            self.fg.start()
            self.gui.pauseButton.setText("Pause")

    # Functions to set the values in the GUI

    def set_frequency(self, freq):
        self.freq = freq
        sfreq = eng_notation.num_to_str(self.freq)
        self.gui.frequencyEdit.setText(QtCore.QString("%1").arg(sfreq))

    def set_gain(self, gain):
        self.gain = gain
        self.gui.gainEdit.setText(QtCore.QString("%1").arg(self.gain))

    def set_bandwidth(self, bw):
        self.bw = bw
        sbw = eng_notation.num_to_str(self.bw)
        self.gui.bandwidthEdit.setText(QtCore.QString("%1").arg(sbw))

    def set_amplifier(self, amp):
        self.amp = amp
        self.gui.amplifierEdit.setText(QtCore.QString("%1").arg(self.amp))

    # Functions called when signals are triggered in the GUI

    def frequencyEditText(self):
        try:
            freq = eng_notation.str_to_num(
                self.gui.frequencyEdit.text().toAscii())
            self.fg.set_frequency(freq)
            self.freq = freq
        except RuntimeError:
            pass

    def gainEditText(self):
        try:
            gain = float(self.gui.gainEdit.text())
            self.fg.set_gain(gain)
            self.gain = gain
        except ValueError:
            pass

    def bandwidthEditText(self):
        try:
            bw = eng_notation.str_to_num(
                self.gui.bandwidthEdit.text().toAscii())
            self.fg.set_bandwidth(bw)
            self.bw = bw
        except ValueError:
            pass

    def amplifierEditText(self):
        try:
            amp = float(self.gui.amplifierEdit.text())
            self.fg.set_amplifier_gain(amp)
            self.amp = amp
        except ValueError:
            pass

    def saveData(self):
        fileName = QtGui.QFileDialog.getSaveFileName(
            self, "Save data to file", ".")
        if(len(fileName)):
            self.fg.save_to_file(str(fileName))

    def dcGainEditText(self):
        gain = float(self.gui.dcGainEdit.text())
        self.fg.set_dc_gain(gain)

    def dcCancelClicked(self, state):
        self.dcGainEditText()
        self.fg.cancel_dc(state)


class my_top_block(gr.top_block):
    def __init__(self, options):
        gr.top_block.__init__(self)

        self.options = options
        self.show_debug_info = True

        self.qapp = QtGui.QApplication(sys.argv)

        self.u = uhd.usrp_source(
            device_addr=options.address, stream_args=uhd.stream_args('fc32'))

        if(options.antenna):
            self.u.set_antenna(options.antenna, 0)

        self.set_bandwidth(options.samp_rate)

        if options.gain is None:
            # if no gain was specified, use the mid-point in dB
            g = self.u.get_gain_range()
            options.gain = float(g.start() + g.stop()) / 2
        self.set_gain(options.gain)

        if options.freq is None:
            # if no freq was specified, use the mid-point
            r = self.u.get_freq_range()
            options.freq = float(r.start() + r.stop()) / 2
        self.set_frequency(options.freq)

        self._fftsize = options.fft_size

        self.snk = qtgui.sink_c(options.fft_size,
                                fft.window.WIN_BLACKMAN_hARRIS,
                                self._freq, self._bandwidth,
                                "UHD Display",
                                True, True, True, False)

        # Set up internal amplifier
        self.amp = blocks.multiply_const_cc(0.0)
        self.set_amplifier_gain(100)

        # Create a single-pole IIR filter to remove DC
        #   but don't connect it yet
        self.dc_gain = 0.001
        self.dc = filter.single_pole_iir_filter_cc(self.dc_gain)
        self.dc_sub = blocks.sub_cc()

        self.connect(self.u, self.amp, self.snk)

        if self.show_debug_info:
            print("Bandwidth: ", self.u.get_samp_rate())
            print("Center Freq: ", self.u.get_center_freq())
            print("Freq Range: ", self.u.get_freq_range())

        # Get the reference pointer to the SpectrumDisplayForm QWidget
        # Wrap the pointer as a PyQt SIP object
        #     This can now be manipulated as a PyQt5.QtGui.QWidget
        self.pysink = sip.wrapinstance(self.snk.qwidget(), QtGui.QWidget)

        self.main_win = main_window(self.pysink, self)

        self.main_win.set_frequency(self._freq)
        self.main_win.set_gain(self._gain)
        self.main_win.set_bandwidth(self._bandwidth)
        self.main_win.set_amplifier(self._amp_value)

        self.main_win.show()

    def save_to_file(self, name):
        self.lock()

        # Add file sink to save data
        self.file_sink = blocks.file_sink(gr.sizeof_gr_complex, name)
        self.connect(self.amp, self.file_sink)

        self.unlock()

    def set_gain(self, gain):
        self._gain = gain
        self.u.set_gain(self._gain)

    def set_frequency(self, freq):
        self._freq = freq
        r = self.u.set_center_freq(freq)

        try:
            self.snk.set_frequency_range(self._freq, self._bandwidth)
        except RuntimeError:
            pass

    def set_bandwidth(self, bw):
        self._bandwidth = bw
        self.u.set_samp_rate(self._bandwidth)

        try:
            self.snk.set_frequency_range(self._freq, self._bandwidth)
        except RuntimeError:
            pass

    def set_amplifier_gain(self, amp):
        self._amp_value = amp
        self.amp.set_k(self._amp_value)

    def set_dc_gain(self, gain):
        self.dc.set_taps(gain)

    def cancel_dc(self, state):
        self.lock()

        if(state):
            self.disconnect(self.u, self.amp)
            self.connect(self.u, (self.dc_sub, 0))
            self.connect(self.u, self.dc, (self.dc_sub, 1))
            self.connect(self.dc_sub, self.amp)
        else:
            self.disconnect(self.dc_sub, self.amp)
            self.disconnect(self.dc, (self.dc_sub, 1))
            self.disconnect(self.u, self.dc)
            self.disconnect(self.u, (self.dc_sub, 0))
            self.connect(self.u, self.amp)

        self.unlock()


def main():
    parser = OptionParser(option_class=eng_option)
    parser.add_option("-a", "--address", type="string", default="addr=192.168.10.2",
                      help="Address of UHD device, [default=%default]")
    parser.add_option("-A", "--antenna", type="string", default=None,
                      help="select Rx Antenna where appropriate")
    parser.add_option("-s", "--samp-rate", type="eng_float", default=1e6,
                      help="set sample rate (bandwidth) [default=%default]")
    parser.add_option("-f", "--freq", type="eng_float", default=2412e6,
                      help="set frequency to FREQ", metavar="FREQ")
    parser.add_option("-g", "--gain", type="eng_float", default=None,
                      help="set gain in dB (default is midpoint)")
    parser.add_option("--fft-size", type="int", default=2048,
                      help="Set number of FFT bins [default=%default]")
    (options, args) = parser.parse_args()

    if len(args) != 0:
        parser.print_help()
        sys.exit(1)

    tb = my_top_block(options)
    tb.start()
    tb.snk.exec_()


if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        pass



# Copyright 2016 Free Software Foundation, Inc.
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-2.0-or-later
#
"""
Class method decorators.
"""

import functools

# pylint: disable=too-few-public-methods,invalid-name


class lazy_property:
    """
    Class method decorator, similar to @property. This causes a method that is
    declared as @lazy_property to be evaluated once, the first time it is called.
    Subsequent calls to this property will always return the cached value.

    Careful! Not suitable for properties that change at runtime.

    Example:

    >>> class Foo:
    ...     def __init__(self):
    ...         self.x = 5
    ...
    ...     @lazy_property
    ...     def xyz(self):
    ...         return complicated_slow_function(self.x)
    ...
    ...
    >>> f = Foo()
    >>> print(f.xyz) # Will give the result, but takes long to compute
    >>> print(f.xyz) # Blazing fast!
    >>> f.x = 7 # Careful! f.xyz will not be updated any more!
    >>> print(f.xyz) # Blazing fast, but returns the same value as before.
    """

    def __init__(self, func):
        self.func = func
        functools.update_wrapper(self, func)

    def __get__(self, instance, owner):
        if instance is None:
            return self
        value = self.func(instance)
        setattr(instance, self.func.__name__, value)
        return value
# pylint: enable=too-few-public-methods,invalid-name


def nop_write(prop):
    """
    Make this a property with a nop setter

    Effectively, makes a property read-only, with no error on write.
    """

    def nop(self, value):
        pass
    return prop.setter(nop)



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import pmt
from gnuradio import gr, gr_unittest, digital, blocks


class test_chunks_to_symbols(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_bc_001(self):
        const = [1 + 0j, 0 + 1j,
                 -1 + 0j, 0 - 1j]
        src_data = (0, 1, 2, 3, 3, 2, 1, 0)
        expected_result = [1 + 0j, 0 + 1j, -1 + 0j, 0 - 1j,
                           0 - 1j, -1 + 0j, 0 + 1j, 1 + 0j]

        src = blocks.vector_source_b(src_data)
        op = digital.chunks_to_symbols_bc(const)

        dst = blocks.vector_sink_c()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        actual_result = dst.data()
        self.assertEqual(expected_result, actual_result)

    def test_bf_002(self):
        const = [-3, -1, 1, 3]
        src_data = (0, 1, 2, 3, 3, 2, 1, 0)
        expected_result = [-3, -1, 1, 3,
                           3, 1, -1, -3]

        src = blocks.vector_source_b(src_data)
        op = digital.chunks_to_symbols_bf(const)

        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        actual_result = dst.data()
        self.assertEqual(expected_result, actual_result)

    def test_bf_2d(self):
        maxval = 4
        dimensions = 2
        const = list(range(maxval * dimensions))
        src_data = [v * 13 % maxval for v in range(maxval)]
        expected_result = []
        for data in src_data:
            for i in range(dimensions):
                expected_result += [const[data * dimensions + i]]

        self.assertEqual(len(src_data) * dimensions, len(expected_result))
        src = blocks.vector_source_b(src_data)
        op = digital.chunks_to_symbols_bf(const, dimensions)

        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        actual_result = dst.data()
        self.assertEqual(expected_result, actual_result)

    def test_bf_3d(self):
        maxval = 48
        dimensions = 3
        const = list(range(maxval * dimensions))
        src_data = [v * 13 % maxval for v in range(maxval)]
        expected_result = []
        for data in src_data:
            for i in range(dimensions):
                expected_result += [const[data * dimensions + i]]

        self.assertEqual(len(src_data) * dimensions, len(expected_result))
        src = blocks.vector_source_b(src_data)
        op = digital.chunks_to_symbols_bf(const, dimensions)

        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        actual_result = dst.data()
        self.assertEqual(expected_result, actual_result)

    def test_bf_100d(self):
        maxval = 48
        dimensions = 100
        const = list(range(maxval * dimensions))
        src_data = [v * 13 % maxval for v in range(maxval)]
        expected_result = []
        for data in src_data:
            for i in range(dimensions):
                expected_result += [const[data * dimensions + i]]

        self.assertEqual(len(src_data) * dimensions, len(expected_result))
        src = blocks.vector_source_b(src_data)
        op = digital.chunks_to_symbols_bf(const, dimensions)

        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        actual_result = dst.data()
        self.assertEqual(expected_result, actual_result)

    def test_ic_003(self):
        const = [1 + 0j, 0 + 1j,
                 -1 + 0j, 0 - 1j]
        src_data = (0, 1, 2, 3, 3, 2, 1, 0)
        expected_result = [1 + 0j, 0 + 1j, -1 + 0j, 0 - 1j,
                           0 - 1j, -1 + 0j, 0 + 1j, 1 + 0j]

        src = blocks.vector_source_i(src_data)
        op = digital.chunks_to_symbols_ic(const)

        dst = blocks.vector_sink_c()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        actual_result = dst.data()
        self.assertEqual(expected_result, actual_result)

    def test_if_004(self):
        const = [-3, -1, 1, 3]
        src_data = (0, 1, 2, 3, 3, 2, 1, 0)
        expected_result = [-3, -1, 1, 3,
                           3, 1, -1, -3]

        src = blocks.vector_source_i(src_data)
        op = digital.chunks_to_symbols_if(const)

        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        actual_result = dst.data()
        self.assertEqual(expected_result, actual_result)

    def test_sc_005(self):
        const = [1 + 0j, 0 + 1j,
                 -1 + 0j, 0 - 1j]
        src_data = (0, 1, 2, 3, 3, 2, 1, 0)
        expected_result = [1 + 0j, 0 + 1j, -1 + 0j, 0 - 1j,
                           0 - 1j, -1 + 0j, 0 + 1j, 1 + 0j]

        src = blocks.vector_source_s(src_data)
        op = digital.chunks_to_symbols_sc(const)

        dst = blocks.vector_sink_c()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        actual_result = dst.data()
        self.assertEqual(expected_result, actual_result)

    def test_sf_006(self):
        const = [-3, -1, 1, 3]
        src_data = (0, 1, 2, 3, 3, 2, 1, 0)
        expected_result = [-3, -1, 1, 3,
                           3, 1, -1, -3]

        src = blocks.vector_source_s(src_data)
        op = digital.chunks_to_symbols_sf(const)

        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        actual_result = dst.data()
        self.assertEqual(expected_result, actual_result)

    def test_sf_callback(self):
        constA = [-3, -1, 1, 3]
        constB = [12, -12, 6, -6]
        src_data = [0, 1, 2, 3, 3, 2, 1, 0]
        expected_result = [12, -12, 6, -6, -6, 6, -12, 12]

        src = blocks.vector_source_s(src_data, False, 1, [])
        op = digital.chunks_to_symbols_sf(constA)
        op.set_symbol_table(constB)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()
        actual_result = dst.data()
        self.assertEqual(expected_result, actual_result)

    def test_sc_callback(self):
        constA = [-3.0 + 1j, -1.0 - 1j, 1.0 + 1j, 3 - 1j]
        constB = [12.0 + 1j, -12.0 - 1j, 6.0 + 1j, -6 - 1j]
        src_data = [0, 1, 2, 3, 3, 2, 1, 0]
        expected_result = [12.0 + 1j, -12.0 - 1j, 6.0 +
                           1j, -6 - 1j, -6 - 1j, 6 + 1j, -12 - 1j, 12 + 1j]

        src = blocks.vector_source_s(src_data, False, 1, [])
        op = digital.chunks_to_symbols_sc(constA)
        op.set_symbol_table(constB)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()
        actual_result = dst.data()
        self.assertEqual(expected_result, actual_result)

    def test_sf_tag(self):
        constA = [-3.0, -1.0, 1.0, 3]
        constB = [12.0, -12.0, 6.0, -6]
        src_data = (0, 1, 2, 3, 3, 2, 1, 0)
        expected_result = [-3, -1, 1, 3,
                           -6, 6, -12, 12]
        first_tag = gr.tag_t()
        first_tag.key = pmt.intern("set_symbol_table")
        first_tag.value = pmt.init_f32vector(len(constA), constA)
        first_tag.offset = 0
        second_tag = gr.tag_t()
        second_tag.key = pmt.intern("set_symbol_table")
        second_tag.value = pmt.init_f32vector(len(constB), constB)
        second_tag.offset = 4

        src = blocks.vector_source_s(
            src_data, False, 1, [
                first_tag, second_tag])
        op = digital.chunks_to_symbols_sf(constB)

        dst = blocks.vector_sink_f()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        actual_result = dst.data()
        self.assertEqual(expected_result, actual_result)

    def test_sc_tag(self):
        constA = [-3.0 + 1j, -1.0 - 1j, 1.0 + 1j, 3 - 1j]
        constB = [12.0 + 1j, -12.0 - 1j, 6.0 + 1j, -6 - 1j]
        src_data = (0, 1, 2, 3, 3, 2, 1, 0)
        expected_result = [-3 + 1j, -1 - 1j, 1 + 1j, 3 - 1j,
                           -6 - 1j, 6 + 1j, -12 - 1j, 12 + 1j]
        first_tag = gr.tag_t()
        first_tag.key = pmt.intern("set_symbol_table")
        first_tag.value = pmt.init_c32vector(len(constA), constA)
        first_tag.offset = 0
        second_tag = gr.tag_t()
        second_tag.key = pmt.intern("set_symbol_table")
        second_tag.value = pmt.init_c32vector(len(constB), constB)
        second_tag.offset = 4

        src = blocks.vector_source_s(
            src_data, False, 1, [
                first_tag, second_tag])
        op = digital.chunks_to_symbols_sc(constB)

        dst = blocks.vector_sink_c()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        actual_result = dst.data()
        self.assertEqual(expected_result, actual_result)


if __name__ == '__main__':
    gr_unittest.run(test_chunks_to_symbols)



#!/usr/bin/env python
#
# Copyright 2020-2021 Daniel Estevez <daniel@destevez.net>
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest, digital, blocks
import numpy as np


class test_nrzi(gr_unittest.TestCase):
    def setUp(self):
        test_size = 256
        self.data = np.random.randint(0, 2, test_size, dtype='uint8')
        self.source = blocks.vector_source_b(self.data, False, 1, [])
        self.sink = blocks.vector_sink_b(1, 0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None
        del(self.data)
        del(self.source)
        del(self.sink)

    def test_encode(self):
        """Performs NRZI encode and checks the result"""
        encoder = digital.diff_encoder_bb(2, digital.DIFF_NRZI)

        self.tb.connect(self.source, encoder, self.sink)
        self.tb.start()
        self.tb.wait()

        expected = np.cumsum((1 ^ self.data) & 1) & 1

        np.testing.assert_equal(
            self.sink.data(), expected,
            'NRZI encode output does not match expected result')

    def test_decode(self):
        """Performs NRZI decode and checks the result"""
        decoder = digital.diff_decoder_bb(2, digital.DIFF_NRZI)

        self.tb.connect(self.source, decoder, self.sink)
        self.tb.start()
        self.tb.wait()

        expected = self.data[1:] ^ self.data[:-1] ^ 1

        np.testing.assert_equal(
            self.sink.data()[1:], expected,
            'NRZI decode output does not match expected result')

    def test_encode_decode(self):
        """Performs NRZI encode and decode and checks the result"""
        encoder = digital.diff_encoder_bb(2, digital.DIFF_NRZI)
        decoder = digital.diff_decoder_bb(2, digital.DIFF_NRZI)

        self.tb.connect(self.source, encoder, decoder, self.sink)
        self.tb.start()
        self.tb.wait()

        np.testing.assert_equal(
            self.sink.data(), self.data,
            'NRZI encoded and decoded output does not match input')


if __name__ == '__main__':
    gr_unittest.run(test_nrzi)



#!/usr/bin/env python
#
# Copyright 2004 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import math
import sys


def gen_approx_table(f, nentries, min_x, max_x):
    """return a list of nentries containing tuples of the form:
    (m, c).  min_x and max_x specify the domain
    of the table.
    """
    r = []
    incx = float(max_x - min_x) / nentries
    for i in range(nentries):
        a = (i * incx) + min_x
        b = ((i + 1) * incx) + min_x
        m = (f(b) - f(a)) / (b - a)
        c = f(a)
        r.append((m, c))
    return r


def scaled_sine(x):
    return math.sin(x * math.pi / 2**31)


def gen_sine_table():
    nbits = 10
    nentries = 2**nbits

    # min_x = -2**31
    # max_x =  2**31-1
    min_x = 0
    max_x = 2**32 - 1
    t = gen_approx_table(scaled_sine, nentries, min_x, max_x)

    # sys.stdout.write ('static const int WORDBITS = 32;\n')
    # sys.stdout.write ('static const int NBITS = %d;\n' % (nbits,))

    # sys.stdout.write ('static const double sine_table[%d][2] = {\n'% (nentries,))

    for e in t:
        sys.stdout.write('  { %22.15e, %22.15e },\n' % (e[0], e[1]))

    # sys.stdout.write ('};\n')


if __name__ == '__main__':
    gen_sine_table()
